#!/usr/bin/python
# -*- coding: utf-8 -*-

"""rna_refinement - RNA refinement with QRNAS.

Models of RNA 3D structures obtained by modeling methods often suffer from local inaccuracies such as clashes or physically improbable bond lengths, backbone conformations, or sugar puckers. To ensure high quality of models, a procedure of re nement should be applied as a  nal step in the modeling pipeline. The software tool QRNAS was developed in our laboratory to perform local re nement of nucleic acid structures based on an extended version of the AMBER force field. The extensions consist of energy terms associated with introduction of explicit hydrogen bonds, idealization of base pair planarity and regularization of backbone conformation.

Read more: Piatkowski, P., Kasprzak, J. M., Kumar, D., Magnus, M., Chojnowski, G., & Bujnicki, J. M. (2016). RNA 3D Structure Modeling by Combination of Template-Based Method ModeRNA, Template-Free Folding with SimRNA, and Refinement with QRNAS. Methods in Molecular Biology (Clifton, N.J.), 1490(Suppl), 217-235. http://doi.org/10.1007/978-1-4939-6433-8_14

Right now, there is 20k steps of refinement.

.. image:: ../pngs/qrnas_0k.png

The initial structure, ``179c48aa-c0d3-4bd6-8e06-12081da22998_ALL_thrs6.20A_clust01-000001_AA.pdb``.

.. image:: ../pngs/qrnas_3k.png

after 3k, ~10min

.. image:: ../pngs/qrnas_10k.png

after 10k steps, around 30min

.. image:: ../pngs/qrnas_20k.png

after 20k steps, around 1h.

**Installation of QRNAS**

Download the QRNAS package from http://genesilico.pl/qrnas/,
unzip the archive, and compile it with the following command::

    ./qrnamake sequential

This should create an executable version of QRNAS.

.. warning:: Please, change the name of the binary file from QRNA to QRNAS!

Be default the script searches QRNAS in <rna-pdb-tools>/opt/qrnas/ .

Usage of QRNA::

    QRNA - Quick Refinement of Nucleic Acids (0.2 alpha)
         by Juliusz Stasiewicz (jstasiewicz@genesilico.pl)

    To use type:
      QRNA -i <input PDBfile> [-o <output PDBfile>] [-c <configfile>] [-p] [-m <restraintsfile>]
    OR specify <input PDBfile>, <output PDBfile> and <restraintsfile> in <configfile> and type just:
      QRNA -c <configfile>

**Installation of this util**

Set up in your bashrc::

   export QRNAS_PATH=<your path to qrnas> # e.g. /home/magnus/src/rna-pdb-tools/opt/qrnas

but default rna-pdb-tools searches for qrnas in <rna-pdb-tools>/opt/qrnas.

**QRNAS at Peyote2**

There is no problem to run QRNAS at our Genesilico cluster, `peyote2`. Tested by mmagnus --170822. Copy files of QRNAS to peyote and run ``./qrnamake sequential``.

To run it at a cluster with the Sun Grid Engine queuing system (this one with qusb ;-))::

     for p in *.pdb; do echo "rna_refinement.py $p >& ${p}.log" | qsub -cwd -V -pe mpi 1 -N "r_$p" ; done

DONE:

- [x] clean up the output structure
- [x] configuration should not be hardcoded
"""
from __future__ import print_function
import argparse
import re
import os
import subprocess
import random
import string

from shutil import copyfile

try:
    PATH = os.environ['RNA_PDB_TOOLS']
except:
    print ('Set up RNA_PDB_TOOLS, see Installation note')
    pass
else:
    QRNAS_PATH = os.getenv('QRNAS_PATH', PATH + '/opt/qrnas/')


class QRNAS:
    """QRNAS"""
    def run(self, inputfile, outputfile, run_in_tmp=False, job_id_random=False, steps=10):
        """Run QRNAS.

        Args:
           inputfile     (str) : path to a input file, use .pdb extensions
           outputfile    (str) : path to on output file
           run_in_tmp    (bool): if yes, run in /tmp otherwise run in currect-directory/tmp/THEjHxilN3nLx2Aj8REg/input
           job_id_random (bool): if yes, then job id will be like, e.g. tmp/gOIFfSdo9tnelFtvs3A7/output.pdb, if now
                                 tmp/output/output.pdb; output, not input because then you can run the same input
                                 a view times and get different outputs (and name of outputs will give
                                 folder names
           steps         (int) : # of steps

        Returns:
           none: works on input/output files

        """
        cwd = os.getcwd()
        # get config
        conftxt = open(QRNAS_PATH + os.sep + 'configfile.txt').read()
        conftxt_tmp = re.sub('\#?\s?NSTEPS.+\d+', 'NSTEPS   ' + str(steps), conftxt)

        if job_id_random:
            JOB_ID = ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(20))
        else:
            JOB_ID = os.path.basename(outputfile.replace('.pdb', ''))

        if run_in_tmp:
            JOB_PATH = '/tmp/' + os.sep + JOB_ID + os.sep
        else:
            JOB_PATH = cwd + os.sep + 'tmp' + os.sep + JOB_ID + os.sep  # run it in place?

        os.makedirs(JOB_PATH)

        # get temp config
        with open(JOB_PATH + os.sep + 'configfile.txt','w') as f:
            f.write(conftxt_tmp)

        # copy input to qrnas folder
        qrnas_inputfile = JOB_PATH + os.path.basename(inputfile)
        qrnas_outputfile = JOB_PATH + os.path.basename(inputfile).replace('.pdb', '.refx.pdb')
        copyfile(inputfile, qrnas_inputfile)

        os.chdir(QRNAS_PATH)
        cmd = './QRNAS -i ' + qrnas_inputfile + \
              ' -c ' + JOB_PATH + 'configfile.txt ' + \
              ' -o ' + qrnas_outputfile
        print(cmd)

        stdout = open(JOB_PATH + 'stdout.txt', 'w')
        stderr = open(JOB_PATH + 'stderr.txt', 'w')
        subprocess.call(cmd, shell=True, stdout=stdout, stderr=stderr)

        os.chdir(cwd)
        print ("Save to %s" % outputfile)
        copyfile(qrnas_outputfile, outputfile)


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-s', '--steps', help="# of steps, default: 20k ", default=20000)
    parser.add_argument('fn', help="input pdb file")
    parser.add_argument('-o', '--output_file', help="output pdb file")
    return parser


# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    q = QRNAS()
    if not args.output_file:
        output_file = args.fn.replace('.pdb', '_refx.pdb')
    else:
        output_file = args.output_file
    q.run(args.fn, output_file, steps=args.steps)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function
import pandas as pd
import argparse
import os


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('csv', help="", default="")
    parser.add_argument('--sep', help="default is ,; can be also '\t'", default=",")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument('--output', help="output csv", default="merged.csv")
    parser.add_argument('--inf', help="inf", default="inf.csv")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    header = 'fn,mqapRNA,rst,'
    if args.inf:
        header += 'inf,mqapRNArst'
    else:
        header += 'mqapRNArst'

    print(header)

    for l in open(args.csv):
        if l.startswith('fn'):
            pass
        else:
            # otherwise there is a problem with float(inf)
            inf = 0

            fn = l.split(',')[0].strip()
            # parser inf.csv file to get inf score
            # search for a file in inf.csv with the same fn as in csv
            if os.path.isfile(args.inf):
                for r in open(args.inf):
                    # target.pdb.outCR,2gdi_tpp-022e50b3-thrs8.00A_clust04X.pdb.outCR,0.848,0.000,0.941,0.000,1.000,0.885,0.000,0.000
                    cells = r.split(',')
                    if not r.startswith('target,fn'):
                        inf_fn = cells[1].replace('.outCR', '')
                        if fn == inf_fn :
                            inf = 1 - float(cells[4]) # inf_WC # 1 -> expressed as a penalty!
                            break

            mq = float(l.split(',')[1].strip())
            exp = 1 - float(l.split(',')[2].strip()) # as penalty right now

            if args.inf:
                mq_exp = float(mq) + exp + float(inf)
            else:
                mq_exp = float(mq) + exp #
            # +(1 * ((1 - exp) * mq))
            # #mq_exp = float(mq) +(1 * ((1 - exp) * mq))
            if args.inf:
                print(','.join([fn, str(mq), str(exp), str(inf), str(mq_exp)]))
            else:
                print(','.join([fn, str(mq), str(exp), str(mq_exp)]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
The output file is <input-file>.ct

Wrapper to

RNAstructure: software for RNA secondary structure prediction and analysis. (2010). RNAstructure: software for RNA secondary structure prediction and analysis., 11, 129. http://doi.org/10.1186/1471-2105-11-129
"""
from __future__ import print_function
import textwrap
import argparse
import subprocess
import tempfile
import six


def exe(cmd):
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err

def dot2ct_file(path, path_output='', verbose=False):
    """
    Args:
        path (str): a path to the input file
        path_output (str): a path to the output file, by default it's '<path>' + '.ct'

    Returns:
        path to the output file
    """
    if not path_output:
        path_output = path + '.ct'
    cmd='dot2ct ' + path + ' ' + path_output + '.ct'
    out, err= exe(cmd)
    if verbose: print(out, end='')
    if not err:
        if verbose: print(' Created: %s' % args.file + '.ct')
    return path_output


def dot2ct(seq, ss, verbose=True):
    """
    Args:
        seq (str|RNAstructure object): sequence
        ss  (str): secondary structure

    Returns:
        ct (str): content of the output ct file
    """
    t = tempfile.NamedTemporaryFile(delete=False)
    with open(t.name, 'w') as f:
        f.write('> ss\n')
        if not isinstance(seq, six.string_types):
            seq = seq.seq
        f.write(seq.strip() + '\n')
        f.write(ss.strip() + '\n')

    cmd='dot2ct ' + t.name + ' ' + t.name + '.ct'
    if verbose: print(cmd)
    out, err = exe(cmd)
    print(out, end='')
    if not err:
        print(' Created: %s' % f + '.ct')
    else:
        print(err)
    with open(t.name + '.ct') as f:
        return f.read().strip()


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawTextHelpFormatter)

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("file", help=textwrap.dedent("""Input is:
>seq
aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
((...((((((((((.......)))))))))).))
"""))
    return parser

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    dot2ct_file(args.file)



    # manage pseudoknots
def x():
    ssl = list(nss)

    pseudoknots = {'A': {'left':'[', 'right':']'},
                   'B': {'left':'{', 'right':'}'},
                   'C': {'left':'<', 'right':'>'},}

    for pseudoknot in pseudoknots:
        pklen = (ssl.count(pseudoknot) / 2)  # to get AAA AAA to get cut middle point to switch
        # from left to right

        if pklen:  # if pk of given type is detected, then run this
            pkindex = 0
            for c, nt in enumerate(ssl):  # s
                #print(nt, pkindex)
                if nt == 'A':
                    if pkindex > (pklen / 2):
                        ssl[c] = pseudoknots[pseudoknot]['right']
                    else:
                        ssl[c] = pseudoknots[pseudoknot]['left']
                    pkindex += 1



    print(xx)
    record = SeqRecord(Seq('-' * 82), id="YP_025292.1", name="HokC", description="toxic membrane protein, small")
    print(record)

    a.append(record)
    print(a)
    AlignIO.write(a, 'a.sto', 'stockholm')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rmsd_calc_trafl - calculate RMSD of transition A->B based on a SimRNA trajectory

After this script, run::

    rna_cal_rmsd_trafl_plot.py rmsd.txt

to get a plot like this:

.. image:: ../pngs/rmsd_transition.png

Prepare structures::

     $ SimRNA -p 17_Das_2_rpr.pdb -n 0 -o 17_Das_2_rpr_n0 # no trafl, trafl will be added
     $ SimRNA -p 5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped.pdb -n 0 -o 5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped
     #(struc must be (~CG~) nope. It has to be a trajectory!)

and run::

     $ rmsd_calc_trafl.py 17_Das_2_rpr.pdb.trafl 17_Das_2_rpr_n0.trafl 5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped_n0.trafl rp17_rmsd.txt
     > calc_rmsd_to_1st_frame
       /Users/magnus/work/opt/simrna/SimRNA_64bitIntel_MacOSX_staticLibs/calc_rmsd_to_1st_frame 17_Das_2_rpr.pdb.trafl 17_Das_2_rpr.pdb_rmsd_e
     < rmsd_out: 17_Das_2_rpr.pdb_rmsd_e
     > struc: 17_Das_2_rpr_n0.trafl 2
     > trafl: 17_Das_2_rpr.pdb.trafl 48
     % saved: 17_Das_2_rpr.pdb.trafl_17_Das_2_rpr_n0.trafl
     > calc_rmsd_to_1st_frame
       /Users/magnus/work/opt/simrna/SimRNA_64bitIntel_MacOSX_staticLibs/calc_rmsd_to_1st_frame 17_Das_2_rpr.pdb.trafl_17_Das_2_rpr_n0.trafl 17_Das_2_rpr.pdb_rmsd_e_17_Das_2_rpr_n0_rmsd_e
     < rmsd_out: 17_Das_2_rpr.pdb_rmsd_e_17_Das_2_rpr_n0_rmsd_e
     > struc: 5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped_n0.trafl 2
     > trafl: 17_Das_2_rpr.pdb.trafl 48
     % saved: 17_Das_2_rpr.pdb.trafl_5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped_n0.trafl
     > calc_rmsd_to_1st_frame
       /Users/magnus/work/opt/simrna/SimRNA_64bitIntel_MacOSX_staticLibs/calc_rmsd_to_1st_frame 17_Das_2_rpr.pdb.trafl_5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped_n0.trafl 17_Das_2_rpr.pdb_rmsd_e_5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped_n0_rmsd_e
     < rmsd_out: 17_Das_2_rpr.pdb_rmsd_e_5k7c_clean_onechain_renumber_as_puzzle_rpr_rmGapped_n0_rmsd_e
      0.000 -695.634
      0.000 -551.093
    xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
     < out: rp17_rmsd.txt

.. warning:: calc_rmsd_to_1st_frame (SimRNA) is required and the path to the binary file is defined in ``config_local``.
"""
from __future__ import print_function
import argparse
import sys
import os
import itertools
import subprocess


class ExceptionRmsdCalcTrafl(Exception):
    pass


# Settings
try:
    from config_local import calc_rmsd_to_1st_frame_exec
except ImportError:
    print("""Create config_local file in the folder of this tool, and set the path to calc_rmsd_to_1st_frame_exec
e.g. calc_rmsd_to_1st_frame_exec="/Users/magnus/work/opt/simrna/SimRNA_64bitIntel_MacOSX_staticLibs/calc_rmsd_to_1st_frame""")


def add_to_trafl(trafl, struc_trafl):
    """Take trafl and struc_trafl and merge it into trafl (out_trafl).

    Args:

         trafl: trafl filename
         struc_trafl: struc_trafl filename (SimRNA trafl file of one structure)

    Returns:

         str: out_trafl, a path to an output file
    """
    print('\nadd_to_trafl')
    struc = open(struc_trafl).read().strip()
    # print struc
    # print struc
    nr_of_files_structure = len(struc.split('\n'))
    assert(nr_of_files_structure == 2,
           'Structure should be in SimRNA trajectory format (with only one frame)')
    print(' input struc: and # of frames (should be only 2!)', struc_trafl, nr_of_files_structure)
    trafl_txt = open(trafl).read().strip()
    print(' add the structure to this trafl: and # of frames', trafl, len(trafl_txt.split('\n')))

    # if len(struc.split('\n')) != len(trafl_txt.split('\n')):
    #    raise Exception('# of atoms in structure != # of atoms in trafl')

    # out_trafl = '/archive/magnus/rmsd_calc_trafl_tmp/' + os.path.basename(trafl) + "_" + os.path.basename(struc_trafl) # temp hack
    # out_trafl = '/archive/magnus/rmsd_calc_trafl_tmp/' + os.path.basename(trafl) + "_" + os.path.basename(struc_trafl) # temp hack
    out_trafl = trafl + "_" + os.path.basename(struc_trafl)
    f = open(out_trafl, 'w')
    f.write(struc + '\n' + trafl_txt)
    f.close()

    print(' saved:', out_trafl)
    return out_trafl


def calc_rmsd_to_1st_frame(trafl):
    """Calc rmsd to the 1st frame of a trafl.

    :param trafl: trafl filename, string
    """
    print('\ncalc_rmsd_to_1st_frame')
    rmsd_out = trafl.replace('.trafl', '_rmsd_e')
    cmd = calc_rmsd_to_1st_frame_exec + ' ' + trafl + ' ' + rmsd_out
    print(' calc_rmsd_to_1st_frame')
    print('  ', cmd)

    o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE,
                         stderr=sys.stdout.flush())  # subprocess.PIPE)
    out = o.stdout.read().strip()
    #err = o.stderr.read().strip()
    # if 'warning' in err:
    #    print('!!! error in reading a trafl', err, out)
    #    sys.exit(0)
    print(' rmsd_out:', rmsd_out)
    return rmsd_out


def head_trafl(trafl, n):
    """ n number of structures, so an output file will have n x 2 lines"""
    fi = open(trafl, 'r')
    trafl_out = trafl.replace('.trafl', '.head' + str(n) + '.trafl')
    fo = open(trafl_out, 'w')
    n = n * 2
    c = 0
    for l in fi:
        fo.write(l)
        c += 1
        if c == n:
            break
    fo.close()
    print(' < saved:', trafl_out)
    return trafl_out


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('trafl', help="trafil")
    parser.add_argument('struc1', help="structure A")
    parser.add_argument('struc2', help="structure B")
    parser.add_argument('rmsds_fn', help="output file")
    return parser


# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    # if you want to head, then uncomment
    #trafl = head_trafl(trafl, 10000000000000000000000)
    calc_rmsd_to_1st_frame(args.trafl)

    trafl = args.trafl
    struc1 = args.struc1
    struc2 = args.struc2
    rmsds_fn = args.rmsds_fn

    # struc 1
    out_trafl = add_to_trafl(trafl, struc1)
    rmsd1_fn = calc_rmsd_to_1st_frame(out_trafl)

    # struc 2
    out_trafl2 = add_to_trafl(trafl, struc2)
    rmsd2_fn = calc_rmsd_to_1st_frame(out_trafl2)

    rmsd1 = open(rmsd1_fn).read().split('\n')
    rmsd2 = open(rmsd2_fn).read().split('\n')

    print(rmsd1[0])
    print(rmsd2[0])

    print('x' * 80)

    c = 0

    fo = open(rmsds_fn, 'w')

    # header
    # fo.write('smk_ready_samsoaked_chainB' + '\t' + 'smk_ready_samsoaked_chainA+SAM' + '\n') #th.basename(struc1).replace(' + '\t' + os.path.basename(struc2) + '\n')
    fo.write(os.path.basename(struc1) + '\t' + os.path.basename(struc2) + '\n')

    for item in itertools.izip(rmsd1, rmsd2):
        x = str(item).split()  # ('  0.000 -502.973', '  0.000 -496.943')
        # print x #["('", '0.000', "-502.973',", "'", '0.000', "-496.943')"]
        if x[0] == "('',":
            break
        r1 = float(x[1])
        r2 = float(x[4])
        # print r1, r2
        if r1 == 0 and r2 == 0:  # skip 0 0
            continue
        fo.write(str(r1) + '\t' + str(r2) + '\n')
        c += 1
    fo.close()
    print(' < out:', rmsds_fn)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""diffpdb - a simple tool to compare text-content of PDB files

The method is quick-and-dirty, but works!

The script takes first 31 characters of lines (or only atom names and residue names)
starting with ``HETATM`` or ``ATOM`` and save these lines to a <filename>.out file.

One file is created per pdb. In the final step DIFF_TOOL is executed
on these two output files. You get a diff output. That's it! Enjoy!

Configuration:

 * ``DIFF_TOOL="open -a diffmerge"`` or ``DIFF_TOOL="kompare"`` to set up what tool would you like to use to diff files in the file ``rna-pdb-tools/tools/diffpdb/diffpdb_conf.py`` (create it if needed)

.. image:: ../../rna_tools/tools/diffpdb/doc/screenshot.png

``./diffpdb.py --names test_data/4/1duq.pdb test_data/4/1duq_decoy0171_amb_clx.pdb``

.. image:: ../../rna_tools/tools/diffpdb/doc/screenshot2.png

and on the Mac (using ``diffmerge``):

.. image:: ../../rna_tools/tools/diffpdb/doc/diffpdb_osx_diffmerge.png

One of the difference that can be detected with the script is variants of atoms.

.. image:: ../../rna_tools/tools/diffpdb/doc/atom-variants.png

or a detection of missing atom.

.. image:: ../../rna_tools/tools/diffpdb/doc/missing-atoms.png

or a detection of missing OP3 at the beginning.

.. image:: ../../rna_tools/tools/diffpdb/doc/missing-op3.png

"""
import sys
import os
import argparse

PATH = os.path.abspath(__file__)
if os.path.islink(PATH):
    PATH = os.path.dirname(os.readlink(PATH)) + '/' # / don't forget
else:
    PATH = os.path.dirname(os.path.abspath(__file__)) + '/'

try:
    from rna_tools.rna_tools_config import DIFF_TOOL
except ImportError:
    DIFF_TOOL = 'diff' ## kompare

def do_file_atom_res(fn):
    """Take a file path, open the file, for each line take
    first 31 characters, saves these lines to a new file."""

    text_new = ''
    for l in open(fn):
        if l.startswith('ATOM') or l.startswith('HETATM') or l.startswith('END') or l.startswith('TER') or l.startswith('MODEL') or l.startswith('ENDMDL'):
            text_new +=  l[12:20].strip() + '\n'
    open(fn + '.out', 'w').write(text_new)

def do_file_atom_res_and_resi(fn):
    """Take a file path, open the file, for each line take
    first 31 characters, saves these lines to a new file."""

    text_new = ''
    for l in open(fn):
        if l.startswith('ATOM') or l.startswith('HETATM') or l.startswith('END') or l.startswith('TER') or l.startswith('MODEL') or l.startswith('ENDMDL'):
            text_new +=  l[12:26].strip() + '\n'
    open(fn + '.out', 'w').write(text_new)
#ATOM   1002  P     A A  48

def do_file(fn):
    """Take a file path, open the file, for each line take
    first 31 characters, saves these lines to a new file."""

    text_new = ''
    for l in open(fn):
        if l.startswith('ATOM') or l.startswith('HETATM') or l.startswith('END') or l.startswith('TER') or l.startswith('MODEL') or l.startswith('ENDMDL'):
            l = l[:31].strip()
            text_new += l + '\n'
    open(fn + '.out', 'w').write(text_new)

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__ + '\n' + '  method: %s ' % DIFF_TOOL, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('--names', help='take only atom residues names', action='store_true')
    parser.add_argument('--names_and_resi', help='take only atom residues names', action='store_true')
    parser.add_argument('--htmlout', help='take only atom residues names', action='store_true')
    parser.add_argument('--method', help='method e.g. `diff`')#, action='store_true')
    parser.add_argument('--verbose', help='be verbose', action='store_true')
    parser.add_argument('f1', help='file')
    parser.add_argument('f2', help='file')
    return parser

if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    str1_fn = args.f1
    str2_fn = args.f2

    if args.method:
        DIFF_TOOL = args.method

    if args.names:
        do_file_atom_res(str1_fn)
        do_file_atom_res(str2_fn)
    elif args.names_and_resi:
        do_file_atom_res_and_resi(str1_fn)
        do_file_atom_res_and_resi(str2_fn)
    else:
        do_file(str1_fn)
        do_file(str2_fn)

    if args.htmlout:
        cmd = 'diff -u %s %s | %s/lib/diff2html.py' % (str1_fn + '.out', str2_fn + '.out', PATH)
        os.system(cmd)
    else:
        cmd = ' '.join([DIFF_TOOL, str1_fn + '.out', str2_fn + '.out'])
        if args.verbose:
            print(cmd)
        os.system(cmd)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from Bio.Seq import Seq
from Bio.Alphabet import generic_dna, generic_protein
import fileinput

for line in fileinput.input():
    seq = Seq(line.strip().replace('-', ''))  # remove gaps
    protein = seq.translate()
    print(protein)
    protein_txt = ''
    index_txt = ''
    for index, p in enumerate(str(protein)):
         protein_txt += '\\' + p + '/'
         index_txt += '' + str(index + 1).ljust(3)
    print(protein_txt)
    print(index_txt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_rosetta_extract_lowscore_decoys.py - a simple wrapper to extract_lowscore_decoys.py

To be used in Jupter notebooks and other scripts.
"""
from __future__ import print_function

import argparse
import subprocess
import os
import logging
import time

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    #parser.add_argument('-', "--", help="", default="")

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("nstruc", help="# of low score structures to obtained", default=100)
    parser.add_argument("file", help="silent file")
    return parser


def rosetta_extract_lowscore_decoys(file, nstruc, log=True):
    cmd = 'extract_lowscore_decoys.py ' + file + ' ' + nstruc
    os.system(cmd)

    if log:
        logging.basicConfig(filename='rna_rosetta_extract_lowscore_decoys.log',level=logging.INFO)
        #logging.basicConfig(level=logging.INFO)
        logging.info(args)
        logging.info(time.strftime("%Y-%m-%d %H:%M"))
        logging.info(args)

    ## p = subprocess.Popen(cmd,
    ##                      stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    ## stderr = p.stderr.read().strip()
    ## if stderr:
    ##     print(stderr)


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    rosetta_extract_lowscore_decoys(args.file, args.nstruc)



#!/usr/bin/env python

"""
Calculate RMSD between two XYZ files

by: Jimmy Charnley Kromann <jimmy@charnley.dk> and Lars Andersen Bratholm <larsbratholm@gmail.com>
project: https://github.com/charnley/rmsd
license: https://github.com/charnley/rmsd/blob/master/LICENSE

"""
import numpy as np
import re
from rna_tools.tools.extra_functions.select_fragment import is_in_selection

def kabsch_rmsd(P, Q):
    """
    Rotate matrix P unto Q and calculate the RMSD
    """
    P = rotate(P, Q)
    return rmsd(P, Q)


def rotate(P, Q):
    """
    Rotate matrix P unto matrix Q using Kabsch algorithm
    """
    U = kabsch(P, Q)

    # Rotate P
    P = np.dot(P, U)
    return P


def kabsch(P, Q):
    """
    The optimal rotation matrix U is calculated and then used to rotate matrix
    P unto matrix Q so the minimum root-mean-square deviation (RMSD) can be
    calculated.

    Using the Kabsch algorithm with two sets of paired point P and Q,
    centered around the center-of-mass.
    Each vector set is represented as an NxD matrix, where D is the
    the dimension of the space.

    The algorithm works in three steps:
    - a translation of P and Q
    - the computation of a covariance matrix C
    - computation of the optimal rotation matrix U

    http://en.wikipedia.org/wiki/Kabsch_algorithm

    Parameters:
    P -- (N, number of points)x(D, dimension) matrix
    Q -- (N, number of points)x(D, dimension) matrix

    Returns:
    U -- Rotation matrix

    """

    # Computation of the covariance matrix
    C = np.dot(np.transpose(P), Q)

    # Computation of the optimal rotation matrix
    # This can be done using singular value decomposition (SVD)
    # Getting the sign of the det(V)*(W) to decide
    # whether we need to correct our rotation matrix to ensure a
    # right-handed coordinate system.
    # And finally calculating the optimal rotation matrix U
    # see http://en.wikipedia.org/wiki/Kabsch_algorithm
    V, S, W = np.linalg.svd(C)
    d = (np.linalg.det(V) * np.linalg.det(W)) < 0.0

    if d:
        S[-1] = -S[-1]
        V[:, -1] = -V[:, -1]

    # Create Rotation matrix U
    U = np.dot(V, W)

    return U


def centroid(X):
    """
    Calculate the centroid from a vectorset X
    """
    C = sum(X)/len(X)
    return C


def rmsd(V, W):
    """
    Calculate Root-mean-square deviation from two sets of vectors V and W.
    """
    D = len(V[0])
    N = len(V)
    rmsd = 0.0
    for v, w in zip(V, W):
        rmsd += sum([(v[i]-w[i])**2.0 for i in range(D)])
    return np.sqrt(rmsd/N)


def get_coordinates(filename, selection, ignore_selection, fmt, ignore_hydrogens):
    """Get coordinates from filename."""
    return get_coordinates_pdb(filename, selection, ignore_selection, ignore_hydrogens)


def get_coordinates_pdb(filename, selection, ignore_selection, ignore_hydrogens):
    """
    Get coordinates from the first chain in a pdb file
    and return a vectorset with all the coordinates.

    """
    # PDB files tend to be a bit of a mess. The x, y and z coordinates
    # are supposed to be in column 31-38, 39-46 and 47-54, but this is not always the case.
    # Because of this the three first columns containing a decimal is used.
    # Since the format doesn't require a space between columns, we use the above
    # column indices as a fallback.
    x_column = None
    V = []
    # Same with atoms and atom naming. The most robust way to do this is probably
    # to assume that the atomtype is given in column 3.
    atoms = []
    resi_set = set()
    with open(filename) as f:
        lines = f.readlines()
        for line in lines:
            # hmm...
            # of models: 490
            # Error: # of atoms is not equal target (1f27_rpr.pdb):641 vs model (struc/1f27_rnakbnm_decoy0001_amb_clx_rpr.pdb):408
            #if line.startswith("TER") or line.startswith("END"):
            #    break
            if line.startswith("ATOM"):
                curr_chain_id = line[21]
                curr_resi = int(line[22:26])
                curr_atom_name = line[12:16].strip()
                if selection:
                    if curr_chain_id in selection:
                        if curr_resi in selection[curr_chain_id]:
                            # ignore if to be ingored (!)
                            #try:
                                    resi_set.add(curr_chain_id + ':' + str(curr_resi))
                                    x = line[30:38]
                                    y = line[38:46]
                                    z = line[46:54]
                                    if ignore_selection:
                                        if not is_in_selection(ignore_selection, curr_chain_id, curr_resi, curr_atom_name):
                                            V.append(np.asarray([x,y,z],dtype=float))
                                    else:
                                        V.append(np.asarray([x,y,z],dtype=float))
                else:
                                    x = line[30:38]
                                    y = line[38:46]
                                    z = line[46:54]
                                    V.append(np.asarray([x,y,z],dtype=float))

    V = np.asarray(V)
    #print filename, resi_set, len(resi_set)
    return len(V), V



#!/usr/bin/env python
"""
Quick reference:

- clarna: show contacts classification of the selected fragment based on ClaRNA
- ss: show secondary structure of the selection based on py3dna.py (3DNA (Lu, Olson 2003))
- ss_all: the same as ss() but for all objects
- pdbsrc: show PDB content (source) of selection.
- seq: show sequence of the selection
- ino: represent ions as sphare and yellow inorganic, such us Mg
- p: shortcut for putting a seq at the bottom. Pretty cool for screenshots with names of objects
- spli: color snRNA of the spliceosome and bases according to identity U(blue), A(orange), G(red), C(forest)
- rp: @todo
- rs: @todo
- rib: @todo
- select br. all within 12 of resi 574
-
If you want more, read for interesting functions https://daslab.stanford.edu/site_data/docs_pymol_rhiju.pdf
"""
# imports
import tempfile
import math
import subprocess
import os
import sys

import getpass
user = getpass.getuser()

try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
    # sys.exit(0)

import imp
try:
    from rna_tools.rna_tools_lib import RNAStructure
    from rna_tools.tools.PyMOL4RNA import code_for_spl
    imp.reload(code_for_spl)
    from rna_tools.tools.PyMOL4RNA import code_for_color_spl
    imp.reload(code_for_color_spl)
    from rna_tools.tools.PyMOL4RNA import code_for_color_spl_objects
    imp.reload(code_for_color_spl_objects)

except ImportError:
    print("rna_tools lib is missing")

    RNA_TOOLS_PATH = ''

try:
    RNA_TOOLS_PATH
    EXECUTABLE
except NameError:
    RNA_TOOLS_PATH = os.environ.get('RNA_TOOLS_PATH')
    EXECUTABLE="/bin/zsh"
    SOURCE=""

try:
    cmd.set('cartoon_gap_cutoff', 0)
except:
    pass

def exe(cmd, verbose=False):
    """Helper function to run cmd. Using in this Python module."""
    if verbose: print('cmd:' + cmd)
    o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                         executable=EXECUTABLE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err

def color_protein():
    cmd.do("color blue, resn ARG+LYS+HIS")
    cmd.do("color red, resn ASP+GLU")
    cmd.do("color green, resn GLY+ALA+VAL+LEU+ILE+MET+PHE")
    cmd.do("color yellow, resn TYR+TRP")
    cmd.do("color forest, resn SER+THR+CYS+ASN+GLN")
    cmd.do("color pink, resn PRO")

cmd.extend('cp', color_protein)

def save_transformed(object, file):
    """Saves the molecule with coordinates from the current orientation.

     Args:
        object (string): PyMOL name
        file (string): a file name to output file

    Example::

         PyMOL>save_transformed 6bk8_RNA_only_Oriented, 6bk8_RNA_only_Oriented.pdb

    Source: <https://pymolwiki.org/index.php/Modeling_and_Editing_Structures>
    """
    m = cmd.get_view(0)
    ttt = [m[0], m[1], m[2], 0.0,
           m[3], m[4], m[5], 0.0,
           m[6], m[7], m[8], 0.0,
           0.0,   0.0,  0.0, 1.0]
    cmd.transform_object(object,ttt,transpose=1)
    cmd.save(file,object)


def color_by_text(txt):
    """Helper function used for color-coding based on residue indexes ranges."""
    for t in txt.strip().split('\n'):
        print(t)
        color, resi = t.replace('color ', '').split(',')
        print((color, resi))
        cmd.color(color.strip(), resi.strip())


def rp():
    """Represent your RNA."""
    cmd.hide("sticks", "all")
    cmd.hide("lines", "all")
    cmd.show("cartoon", "all")
    cmd.set("cartoon_ring_mode", 3)
    cmd.set("cartoon_ring_finder", 2)
    cmd.set("cartoon_ladder_mode", 1)

def show_all_at_once():
    cmd.set('states', 'on')


def rp06():
  txt = """color black, all
  color pink, resi 2-10+163-170
  color grey, resi 12-33
  color green, resi 40-41
  color green, resi 161-162
  color orange, resi 45-61
  color green, resi 64-73
  color blue, resi 74-155
  color cyan, resn B1Z"""
  for t in txt.split('\n'):
    color, resi = t.replace('color ', '').split(',')
    print(color, resi)
    cmd.color(color.strip(), resi.strip())


def grid_on():
    cmd.set('grid_mode', 1)


def grid_off():
    cmd.set('grid_mode', 0)


def rp14():
  """color black; # everything
 color blue, resi 1-5+55-59; # p1
 color green, resi 7-11+16-20; # p2
 color magenta, resi 23+60; # pk
 color yellow, resi 29-34+45-50; # p3
 color grey, resi 24-28+51-54; # e-loop
 color red, resi 6+21+22+24+25+28+52+54; # higly conserved"""
 #color blue, resi 5+55


  txt ="""color black, all
 color red, resi 1-5+55-59
 color blue, resi 1-5+55-59; # p1
 color green, resi 7-11+16-20
 color magenta, resi 23+60
 color yellow, resi 29-34+45-50
 color grey, resi 24-28+51-54
 color red, resi 6+21+22"""
  for t in txt.split('\n'):
    color, resi = t.replace('color ', '').split(',')
    print(color, resi)
    cmd.color(color.strip(), resi.strip())

def rp14s():
  """color with Baker's SHAPE data for rp14!"""
  txt = """
   color yellow, resi 12-15+25-29+35-44
   color red, resi 21-24+53+54+60
  """
  color_by_text(txt)

def rs():
    """    The function creates super-cool cartoon-like RNA and colors each structure as a rainbow.
    Good to view aligned structures in a grid.

    .. image:: ../../rna_tools/utils/PyMOL4RNA/doc/rs.png
    """
    cmd.hide("sticks", "all")
    cmd.hide("lines", "all")
    cmd.show("cartoon", "all")
    cmd.set("cartoon_ring_mode", 3)
    cmd.set("cartoon_ring_finder", 2)
    cmd.set("cartoon_ladder_mode", 2)
    cmd.set("cartoon_ring_transparency", 0.30)
    cmd.spectrum()

    obj_list = cmd.get_names('objects')

    colours = ['rainbow']
    ncolours = len(colours)
    # Loop over objects
    i = 0
    for obj in obj_list:
        print("  ", obj, colours[i])
        cmd.spectrum('count', colours[i], obj)
        i = i+1
        if(i == ncolours):
           i = 0


def rcomp():
    """RNA like in papers ;-)

    Similar to rc() but this time it colors each (and every) structure in different colour.
    Great on viewing-comparing superimposed structures.

    """
    cmd.hide("sticks", "all")
    cmd.hide("lines", "all")
    cmd.show("cartoon", "all")
    cmd.set("cartoon_ring_mode", 3)
    cmd.set("cartoon_ring_finder", 2)
    cmd.set("cartoon_ladder_mode", 2)
    cmd.set("cartoon_ring_transparency", 0.30)

    obj_list = cmd.get_names('objects')

    colours = ['red', 'green', 'blue', 'yellow', 'violet', 'cyan',    \
           'salmon', 'lime', 'pink', 'slate', 'magenta', 'orange', 'marine', \
           'olive', 'purple', 'teal', 'forest', 'firebrick', 'chocolate',    \
           'wheat', 'white', 'grey' ]
    ncolours = len(colours)

           # Loop over objects
    i = 0
    for obj in obj_list:
        print("  ", obj, colours[i])
        cmd.color(colours[i], obj)
        i = i+1
        if(i == ncolours):
           i = 0


def align_all( subset = [] ):
  """
  Superimpose all open models onto the first one.
  This may not work well with selections.

  This function is probably taken from https://daslab.stanford.edu/site_data/docs_pymol_rhiju.pdf
  """
  print("""This returns a list with 7 items:

    RMSD after refinement
    Number of aligned atoms after refinement
    Number of refinement cycles
    RMSD before refinement
    Number of aligned atoms before refinement
    Raw alignment score
    Number of residues aligned """)

  AllObj=cmd.get_names("all")
  for x in AllObj[1:]:
    #print(AllObj[0],x)
    subset_tag = ''
    if isinstance( subset, int ):
      subset_tag = ' and resi %d' % subset
    elif isinstance( subset, list ) and len( subset ) > 0:
      subset_tag = ' and resi %d' % (subset[0])
      for m in range( 1,len(subset)): subset_tag += '+%d' % subset[m]
    elif isinstance( subset, str ) and len( subset ) > 0:
      subset_tag = ' and %s' % subset
    values = cmd.align(x+subset_tag,AllObj[0]+subset_tag)
    print(AllObj[0], x, ' '.join([str(v) for v in values]), '-- RMSD', values[3], ' of ', values[6], 'residues')
    cmd.zoom()


def pdb():
    """Get PDB content of selection.

    .. image:: ../../rna_tools/utils/PyMOL4RNA/doc/pdb.png"""
    tmpfn = '/tmp/pymol_get_pdb.pdb'
    cmd.save(tmpfn, '(sele)')
    s = RNAStructure(tmpfn)
    for l in s.lines:
        print(l)


def x3dna():
    f = tempfile.NamedTemporaryFile(delete=False) # True)
    #cmd.save(f.name + '.pdb', '(backbone_)')
    cmd.save(f.name + '.pdb', '(sele)')
    out, err = exe("rna_x3dna.py --show-log " + f.name + ".pdb ")
    print('\n'.join(out.split('\n')[1:]))  # to remove first line of py3dna /tmp/xxx
    if err:
        print(err)
    f.close()

def clarna():
    """Get contacts classification of the selected fragment based on ClaRNA.

    .. image:: ../../rna_tools/tools/PyMOL4RNA/doc/clarna.png
    """
    f = tempfile.NamedTemporaryFile(delete=False) # True)
    #cmd.save(f.name + '.pdb', '(backbone_)')
    cmd.save(f.name + '.pdb', '(sele)')
    out, err = exe(SOURCE + " && " + CLARNA_RUN + " -ipdb " + f.name + '.pdb -bp+stack')
    print('\n'.join(out.split('\n')[1:]))  # to remove first line of py3dna /tmp/xxx
    if err:
        print(err)
    f.close()


def seq():
    """Get sequence of the selected fragment using ``rna_pdb_toolsx.py --get_seq ``.

    .. image:: ../../rna_tools/utils/PyMOL4RNA/doc/ss.png
    """
    f = tempfile.NamedTemporaryFile(delete=False) # True)
    cmd.save(f.name, '(sele)')
    out, err = exe('source ~/.zshrc && ' + RNA_TOOLS_PATH + '/bin/rna_pdb_toolsx.py --get_seq ' + f.name)
    print(out)
    if err:
        print(err)
    f.close()

def ss():
    """Get Secondary Structure of (sele) based on py3dna.py.

    .. image:: ../../rna_tools/utils/PyMOL4RNA/doc/ss.png
    """
    f = tempfile.NamedTemporaryFile(delete=False) # True)
    cmd.save(f.name, '(sele)')
    out, err = exe(RNA_TOOLS_PATH + '/bin/rna_x3dna.py ' + f.name)
    print('\n'.join(out.split('\n')[2:]))  # to remove first line of py3dna /tmp/xxx
    if err:
        print(err)
    f.close()


def ss_all():
    """The same as ss() but for all objects."""
    subset = "*"
    AllObj = cmd.get_names("all")
    # print AllObj
    for name in AllObj[:]:
        if not name.startswith('_align'):
            print('> ' + name)
            f = tempfile.NamedTemporaryFile(delete=False) # True)
            cmd.save(f.name, name)
            out, err = exe(RNA_TOOLS_PATH + '/bin/rna_x3dna.py ' + f.name)
            print('\n'.join(out.split('\n')[2:]))  # to remove first line of py3dna /tmp/xxx
            # hide this line: is >tmpGCszi7 nts=4 [tmpGCszi7] -- secondary structure derived by DSSR
            if err:
                print(err)
            f.close()
    print('-- secondary structure derived by DSSR')


def p():
    """A shortcut for putting a seq at the bottom. Pretty cool for screenshots with names of objects.

    .. image:: ../../rna_tools/utils/PyMOL4RNA/doc/p.png
    """
    cmd.set("seq_view_format", 4)
    cmd.set("seq_view", 1)
    cmd.set("seq_view_location", 1)
    cmd.set("seq_view_overlay", 1)


def rna_cartoon():
    """http://www-cryst.bioc.cam.ac.uk/members/zbyszek/figures_pymol

    .. image:: ../pngs/rna_cartoon.png
    """
    cmd.set("cartoon_ring_mode", 3)
    cmd.set("cartoon_ring_finder", 1)
    cmd.set("cartoon_ladder_mode", 1)
    cmd.set("cartoon_nucleic_acid_mode", 4)
    cmd.set("cartoon_ring_transparency", 0.5)


def rp17():
    """Color-coding for secondary structure elements for the RNA Puzzle 17.

    For the variant::

         CGUGGUUAGGGCCACGUUAAAUAGUUGCUUAAGCCCUAAGCGUUGAUAAAUAUCAGGUGCAA
         ((((.[[[[[[.))))........((((.....]]]]]]...(((((....)))))..))))
         # len 62-nt

    .. image:: ../../rna_tools/tools/PyMOL4RNA/doc/rna.png
    """
    txt = """color forest, resi 1-5+12-16; # p1
 color magenta, resi 6-11+34-39;
 color grey, resi 17-24;
 color marine, resi 25-28+59-62;
 color deepblue, resi 29-33+40-42;
 color orange, resi 44-47+48-56;
 color yellow, resi 57-58;
 color red, resi 19+20+21;
"""
    color_by_text(txt)

def rp17csrv():
    """Color-coding for secondary structure elements for the RNA Puzzle 17.

    For the variant::

         CGUGGUUAGGGCCACGUUAAAUAGUUGCUUAAGCCCUAAGCGUUGAUAAAUAUCAGGUGCAA
         ((((.[[[[[[.))))........((((.....]]]]]]...(((((....)))))..))))
         # len 62-nt

    .. image:: ../../rna_tools/utils/PyMOL4RNA/doc/rna.png
    """
    txt = """color forest, resi 1-5+12-16; # p1
 color magenta, resi 6-11+34-39;
 color grey, resi 17-24;
 color marine, resi 25-28+59-62;
 color deepblue, resi 29-33+40-42;
 color orange, resi 44-47+48-56;
 color yellow, resi 57-58;
 color red, resi 5+19+20+21+31+32+33+40+41+42
"""
    color_by_text(txt)



def rp172():
    """Color-coding for secondary structure elements for the RNA Puzzle 17.

    For the variant::

         CGUGGUUAGGGCCACGUUAAAUAGUUGCUUAAGCCCUAAGCGUUGAUAUCAGGUGCAA
         ((((.[[[[[[.))))........((((.....]]]]]]...((((()))))..))))
         # len 58-nt

    See rp17()
    """

    txt = """color forest, resi 1-5+12-16; # p1
 color magenta, resi 6-11+34-39
 color grey, resi 17-24
 color marine, resi 25-28+55-58
 color deepblue, resi 29-33+40-42;
 color orange, resi 43-47+48-52;
 color yellow, resi 53-54;
 color red, resi 19+20+21;
"""
    color_by_text(txt)

def color_aa_types():
    """Color aminoacides types like in Cider (http://pappulab.wustl.edu/CIDER/)"""
    txt = """
color gray70, resn Ala+Ile+Leu+Met+Phe+Trp+Val #hydrophobic
color yellow, resn Tyr+Trp #aromatic
color blue, resn Arg+Lys+His # positive
color forest, resn GLN+SER+GLY+thr
color pink, resn PRO # pro
color red, resn GLU+asp # """
    print("""color (according to) amino-acids types)
hydrohobic (gray)  Ala+Ile+Leu+Met+Phe+Trp+Val
aromatic (yellow) Tyr+Trp
positive (blue)  Arg+Lys+His
polar (forest) Gln+Ser+Glu+Thr
negative (red) Glu+Asp
prolina ;) (pink) Pro""")
    color_by_text(txt)


def color_obj(rainbow=0):

        """
        stolen from :)
AUTHOR
        Gareth Stockwell

USAGE
        color_obj(rainbow=0)

        This function colours each object currently in the PyMOL heirarchy
        with a different colour.  Colours used are either the 22 named
        colours used by PyMOL (in which case the 23rd object, if it exists,
        gets the same colour as the first), or are the colours of the rainbow

        """

        # Process arguments
        rainbow = int(rainbow)

        # Get names of all PyMOL objects
        obj_list = cmd.get_names('objects')

        if rainbow:

           print("\nColouring objects as rainbow\n")

           nobj = len(obj_list)

           # Create colours starting at blue(240) to red(0), using intervals
           # of 240/(nobj-1)
           for j in range(nobj):
              hsv = (240-j*240/(nobj-1), 1, 1)
              # Convert to RGB
              rgb = hsv_to_rgb(hsv)
              # Define the new colour
              cmd.set_color("col" + str(j), rgb)
              print(obj_list[j], rgb)
              # Colour the object
              cmd.color("col" + str(j), obj_list[j])

        else:
           # List of available colours
           colours = ['red', 'green', 'blue', 'yellow', 'violet', 'cyan',    \
           'salmon', 'lime', 'pink', 'slate', 'magenta', 'orange', 'marine', \
           'olive', 'purple', 'teal', 'forest', 'firebrick', 'chocolate',    \
           'wheat', 'white', 'grey' ]
           ncolours = len(colours)

           # Loop over objects
           i = 0
           for obj in obj_list:
              print("  ", obj, colours[i])
              cmd.color(colours[i], obj)
              i = i+1
              if(i == ncolours):
                 i = 0


def names():
    # Get names of all PyMOL objects
    obj_list = cmd.get_names('objects')
    for o in obj_list:
        print(o)


def color_rbw(rainbow=0):
        """
        similar to color_obj() but this time colors every obect as rainbow
        """
        rainbow = int(rainbow)

        # Get names of all PyMOL objects
        obj_list = cmd.get_names('objects')

        if rainbow:

           print("\nColouring objects as rainbow\n")

           nobj = len(obj_list)

           # Create colours starting at blue(240) to red(0), using intervals
           # of 240/(nobj-1)
           for j in range(nobj):
              hsv = (240-j*240/(nobj-1), 1, 1)
              # Convert to RGB
              rgb = hsv_to_rgb(hsv)
              # Define the new colour
              cmd.set_color("col" + str(j), rgb)
              print(obj_list[j], rgb)
              # Colour the object
              cmd.color("col" + str(j), obj_list[j])
        else:
           colours = ['rainbow']
           ncolours = len(colours)

           # Loop over objects
           i = 0
           for obj in obj_list:
              print("  ", obj, colours[i])
              cmd.spectrum('count', colours[i], obj)
#              cmd.color(colours[i], obj)
              i = i+1
              if(i == ncolours):
                 i = 0

def ino():
    """Sphare and yellow inorganic, such us Mg.

    .. image:: ../../rna_tools/utils/PyMOL4RNA/doc/ion.png"""
    cmd.show("spheres", "inorganic")
    #cmd.set('sphere_scale', '0.25', '(all)')
    cmd.set('sphere_scale', '1', '(all)')
    cmd.color("yellow", "inorganic")

mapping = [[u'PRP8', 'A', u'skyblue'], [u'BRR2', 'B', u'grey60'], [u'BUD31', 'C', u'dirtyviolet'], [u'CEF1', 'D', u'raspberry'], [u'CLF1', 'E', u'raspberry'], [u'CWC15', 'F', u'dirtyviolet'], [u'CWC16/YJU2', 'G', u'lightteal'], [u'CWC2', 'H', u'ruby'], [u'CWC21', 'I', u'violetpurple'], [u'CWC22', 'J', u'bluewhite'], [u'CWC25', 'K', u'deepteal'], [u'Intron', 'L', u'black'], [u'ISY1', 'M', u'dirtyviolet'], [u'LEA1', 'N', u'palegreen'], [u'Msl1', 'O', u'palegreen'], [u'PRP45', 'P', u'lightpink'], [u'PRP16', 'Q', u'smudge'], [u'CDC40\xa0(PRP17, SLU4, XRS2)', 'R', u'dirtyviolet'], [u'PRP19 (PSO4)', 'S', u'grey70'], [u'PRP46', 'T', u'lightblue'], [u'SLT11/ECM2', 'U', u'chocolate'], [u'SNT309', 'V', u'grey70'], [u'SNU114', 'W', u'slate'], [u'SYF2', 'X', u'brightorange'], [u'SYF1', 'Y', u'brightorange'], [u'U2', 'Z', u'forest'], [u'U5', 'a', u'density'], [u'U5_SmRNP', 'b', u'deepblue'], [u'U6', 'c', u'firebrick'], [u'Intron', 'r', u'grey50'], [u'Exon', 'z', u'yellow'], [u'exon-3', 'y', u'yellow'], [u'exon-5', 'z', u'yellow'], [u'PRP4 ', 'd', u'grey50'], [u'PRP31', 'e', u'grey50'], [u'PRP6', 'f', u'grey50'], [u'PRP3', 'g', u'grey50'], [u'DIB1', 'h', u'grey50'], [u'SNU13', 'i', u'grey50'], [u'LSM8', 'j', u'grey50'], [u'LSM2', 'k', u'grey50'], [u'LSM3', 'l', u'grey50'], [u'LSM6', 'm', u'grey50'], [u'LSM5', 'n', u'grey50'], [u'LSM7', 'o', u'grey50'], [u'LSM4', 'p', u'grey50'], [u'SNU66', 'q', u'grey50'], [u'RNA (intron or U6 snRNA)', 'r', u'grey50'], [u'5EXON', 's', u'grey50'], [u'BUD13', 't', u'grey60'], [u'CLF2', 'u', u'rasberry'], [u'Cus1', 'v', u'palegreen'], [u'CWC24', 'w', u'grey60'], [u'CWC27', 'x', u'grey60'], [u'HSH155', '1', u'smudge'], [u'HSH49', '2', u'sand'], [u'PML1', '3', u'grey60'], [u'PRP11', '4', u'palegreen'], [u'PRP2', '5', u'palegreen'], [u'RDS3', '6', u'palegreen'], [u'RSE1', '7', u'smudge'], [u'SNU17', '8', u'grey60'], [u'Ysf3', '9', u'palegreen'], [u'cwc23', 'd', u'grey50'], [u'SPP382\xa0(CCF8, NTR1)', 'e', u'grey50'], [u'NTR2', 'f', u'grey50'], [u'PRP43', 'g', u'grey50'], [u'SMB1', 'h', u'grey50'], [u'SME1', 'i', u'grey50'], [u'SMX3', 'j', u'grey50'], [u'SMX2\xa0(SNP2)', 'k', u'grey50'], [u'SMD3', 'l', u'grey50'], [u'SMD1', 'm', u'grey50'], [u'SMD2', 'n', u'grey50'], [u'PRP22', 'o', u'grey50'], [u'PRP18', 'p', u'grey50'], [u'SLU7', 'q', u'grey50'], [u'SMF', 'd', u'grey50'], [u'SMG', 'e', u'grey50'], [u'PRP9', 'f', u'grey50'], [u'PRP21', 'g', u'grey50'], [u'SNU23', 'r', u'grey50'], [u'PRP38', 's', u'grey50'], [u'SPP381', 'w', u'grey50']]

def spl(arg=''):
    """
    action='', name=''
    """
    if ' ' in arg:
        action, name = arg.split()
        name = name.lower()
    else:
        action = arg
        name = ''
    #import pandas as pd
    #df = pd.read_excel("/home/magnus/Desktop/pyMoL_colors-EMX.xlsx")
    if not action or action == 'help':
        spl_help()
    elif action == 'color' or arg=='c':
        code_for_color_spl.spl_color()
        code_for_color_spl_objects.spl_color()
    elif arg == 'extract all' or arg == 'e':
        code_for_spl.spl_extract()
    elif arg == 'align' or arg=='a':
        cmd.do("""
    align /5gm6//6, /5lj3//V;
    align /5mps//6, /5lj3//V;
    align /6exn//6, /5lj3//V;
    align /5y88//D, /5lj3//V;
    align /5ylz//D, /5lj3//V;
    """)
    else:
        spl_help()

cmd.extend('spl', spl)

def spl_help():
    print("""################ SPL #################
extract all (ea)  - show
colors            - list all colors
######################################
""")

spl_help()


def __spl_color():
    for m in mapping:
        protein = m[0]
        chain = m[1]
        color = m[2]
        print('\_' + ' '.join([protein, chain, color]))
        cmd.do('color ' + color + ', chain ' + chain)
        # cmd.do('color firebrick, chain V') # U6

def _spl_color():
    """Color spl RNAs (for only color spl RNA and use 4-color code for residues see `spl2`)
    """
    AllObj = cmd.get_names("all")
    for name in AllObj:
        if 'Exon' in name or 'exon' in name:
            cmd.color('yellow', name)
        if 'Intron' in name or 'intron' in name or '5splicing-site' in name:
            cmd.color('gray40', name)
        if '3exon-intron' in name.lower():
            cmd.color('gray20', name)
        if name.startswith("U2_snRNA"):
            cmd.color('forest', name)
        if name.startswith("U5_snRNA"):
            cmd.color('blue', name)
        if name.startswith("U4_snRNA"):
            cmd.color('orange', name)
        if name.startswith("U6_snRNA"):
            cmd.color('red', name)

    cmd.do('color gray')

    # trisnrp
    cmd.do('color orange, chain V') # conflict
    cmd.do('color red, chain W')
    cmd.do('color blue, chain U')
    #
    cmd.do('color blue, chain 5')
    cmd.do('color forest, chain 2')
    cmd.do('color red, chain 6')
    cmd.do('color orange, chain 4')
    cmd.do('color yellow, chain Y')
    # shi
    cmd.do('color blue, chain D') # u5
    cmd.do('color forest, chain L') # u2
    cmd.do('color red, chain E') # u6
    cmd.do('color yellow, chain M')
    cmd.do('color yellow, chain N')
    # afte branch
    cmd.do('color blue, chain U') # u5
    cmd.do('color forest, chain Z') # u2
    cmd.do('color red, chain V') # u6
    cmd.do('color yellow, chain E')
    cmd.do('color black, chain I')
    # 5WSG
    # Cryo-EM structure of the Catalytic Step II spliceosome (C* complex) at 4.0 angstrom resolution
    cmd.do('color blue, chain D') # u5
    #cmd.do('color forest, chain L') # u2
    cmd.do('color yellow, chain B')
    cmd.do('color yellow, chain b')
    cmd.do('color black, chain N')
    cmd.do('color black, chain M')

    cmd.do('color black, chain 3') # orange
    cmd.do('color black, chain E') # yellow
    cmd.do('color black, chain i')
    cmd.do('color black, chain e')

    cmd.do('color black, chain e')

    cmd.do('color dirtyviolet, chain L') # bud31
    cmd.do('color rasberry, chain L') # CERF1

    cmd.do('color skyblue, chain A') # PRP8
    cmd.do('color grey60, chain B') # BRR2
    cmd.do('color dirtyiolet, chain L') # BUD31
    cmd.do('color rasberry, chain O') # CEF1
    cmd.do('color rasberry, chain S') # CLF1
    cmd.do('color dirtyviolet, chain P') # CWC15
    cmd.do('color lightteal, chain D') # CWC16/YJU2
    cmd.do('color ruby, chain M') # CWC2
    cmd.do('color violetpurple, chain R') # CWC21
    cmd.do('color bluewhite, chain H') # CWC22
    cmd.do('color deepteal, chain F') # CWC25
    cmd.do('color black, chain I') # Intron
    cmd.do('color dirtyviolet, chain G') # ISY1
    cmd.do('color palegreen, chain W') # LEA1
    cmd.do('color palegreen, chain Y') # Msl1
    cmd.do('color lightpink, chain K') # PRP45
    cmd.do('color smudge, chain Q') # Prp16
    cmd.do('color grey70, chain t') # Prp19
    cmd.do('color lightblue, chain J') # PRP46
    cmd.do('color chocolate, chain N') # SLT11/ECM2
    cmd.do('color grey70, chain s') # Snt309
    cmd.do('color slate, chain C') # SNU114
    cmd.do('color brightorange, chain T') # SYF1
    cmd.do('color forest, chain Z') # U2
    cmd.do('color density, chain U') # U5
    cmd.do('color deepblue, chain b') # U5_Sm

    cmd.do('bg gray')
    # cmd.do('remove (polymer.protein)')

    cmd.set("cartoon_tube_radius", 1.0)
    ino()

def spl2():
    """Color spl RNAs and use 4-color code for residues (for only color spl RNA see `spl`)
    """

    AllObj = cmd.get_names("all")
    for name in AllObj:
        if 'Exon' in name or 'exon' in name:
            cmd.color('yellow', name)
        if 'Intron' in name or 'intron' in name or '5splicing-site' in name:
            cmd.color('gray40', name)
        if '3exon-intron' in name.lower():
            cmd.color('gray20', name)
        if name.startswith("U2_snRNA"):
            cmd.color('forest', name)
        if name.startswith("U5_snRNA"):
            cmd.color('blue', name)
        if name.startswith("U4_snRNA"):
            cmd.color('orange', name)
        if name.startswith("U6_snRNA"):
            cmd.color('red', name)

    cmd.do('color gray')

    # trisnrp
    cmd.do('color orange, chain V') # conflict
    cmd.do('color red, chain W')
    cmd.do('color blue, chain U')
    #
    cmd.do('color blue, chain 5')
    cmd.do('color forest, chain 2')
    cmd.do('color red, chain 6')
    cmd.do('color orange, chain 4')
    cmd.do('color yellow, chain Y')
    # shi
    cmd.do('color blue, chain D') # u5
    cmd.do('color forest, chain L') # u2
    cmd.do('color red, chain E') # u6
    cmd.do('color yellow, chain M')
    cmd.do('color yellow, chain N')
    # afte branch
    cmd.do('color blue, chain U') # u5
    cmd.do('color forest, chain Z') # u2
    cmd.do('color red, chain V') # u6
    cmd.do('color yellow, chain E')
    cmd.do('color black, chain I')
    # 5WSG
    # Cryo-EM structure of the Catalytic Step II spliceosome (C* complex) at 4.0 angstrom resolution
    cmd.do('color blue, chain D') # u5
    #cmd.do('color forest, chain L') # u2
    cmd.do('color yellow, chain B')
    cmd.do('color yellow, chain b')
    cmd.do('color black, chain N')
    cmd.do('color black, chain M')

    cmd.do('color black, chain 3') # orange
    cmd.do('color black, chain E') # yellow
    cmd.do('color black, chain i')
    cmd.do('color black, chain e')

    cmd.do('bg gray')
    cmd.do('remove (polymer.protein)')

    cmd.color("red",'resn rG+G and name n1+c6+o6+c5+c4+n7+c8+n9+n3+c2+n1+n2')
    cmd.color("forest",'resn rC+C and name n1+c2+o2+n3+c4+n4+c5+c6')
    cmd.color("orange",'resn rA+A and name n1+c6+n6+c5+n7+c8+n9+c4+n3+c2')
    cmd.color("blue",'resn rU+U and name n3+c4+o4+c5+c6+n1+c2+o2')
    cmd.set("cartoon_tube_radius", 1.0)
    ino()


def _spli():
    """
    # this trick is taken from Rhiju's Das code
    color red,resn rG+G and name n1+c6+o6+c5+c4+n7+c8+n9+n3+c2+n1+n2
    color forest,resn rC+C and name n1+c2+o2+n3+c4+n4+c5+c6
    color orange, resn rA+A and name n1+c6+n6+c5+n7+c8+n9+c4+n3+c2
    color blue, resn rU+U and name n3+c4+o4+c5+c6+n1+c2+o2

    #
    #cmd.color("yellow", "*intron*")
    #cmd.color("yellow", "*exon*")

    #cmd.show("spheres", "inorganic")
    #cmd.color("yellow", "inorganic")
    """
    cmd.color("orange", "U4_snRNA*")
    cmd.color("red", "U6_snRNA*")
    cmd.color("blue", "U5_snRNA*")
    cmd.color("green", "U2_snRNA*")
    cmd.color("red",'resn rG+G and name n1+c6+o6+c5+c4+n7+c8+n9+n3+c2+n1+n2')
    cmd.color("forest",'resn rC+C and name n1+c2+o2+n3+c4+n4+c5+c6')
    cmd.color("orange",'resn rA+A and name n1+c6+n6+c5+n7+c8+n9+c4+n3+c2')
    cmd.color("blue",'resn rU+U and name n3+c4+o4+c5+c6+n1+c2+o2')


def g2():
    txt = """color gray, all;
    color yellow, chain B;
    color red, resi 788-824;
    color yellow, resi 828-866;
    color forest, resi 476-490;
    """
    color_by_text(txt)
    cmd.color('pink', 'resi 120')
    cmd.color('pink', 'resi 2')


def rgyration(selection='(all)', quiet=1):
    '''

[PyMOL] RES: radius of gyration
From: Tsjerk Wassenaar <tsjerkw@gm...> - 2011-03-31 14:07:03
https://sourceforge.net/p/pymol/mailman/message/27288491/
DESCRIPTION

    Calculate radius of gyration

USAGE

    rgyrate [ selection ]
 :::warning:::
 if nothing is selected  function is calculating radius of gyration for all pdbs in current Pymol session
    '''
    from itertools import izip
    quiet = int(quiet)
    model = cmd.get_model(selection).atom
    x = [i.coord for i in model]
    mass = [i.get_mass() for i in model]
    xm = [(m*i,m*j,m*k) for (i,j,k),m in izip(x,mass)]
    tmass = sum(mass)
    rr = sum(mi*i+mj*j+mk*k for (i,j,k),(mi,mj,mk) in izip(x,xm))
    mm = sum((sum(i)/tmass)**2 for i in izip(*xm))
    rg = math.sqrt(rr/tmass - mm)
    if not quiet:
        print("Radius of gyration: %.2f" % (rg))
    return rg


def qrnass():
    cmd.save('sele.pdb', '(sele)')
    mini('sele.pdb')


def qrnas():
    subset = "*"
    AllObj=cmd.get_names("all")
    #print AllObj
    for x in AllObj[:]:
      print(x, 'qrnas...')
      #print(AllObj[0],x)
      f = tempfile.NamedTemporaryFile(delete=True)
      #print f.name
      #f.write(XX)
      cmd.save(f.name, x)
      #p = Process(target=mini)
      #p.start()
      mini()
      #cmd.load('out.pdb', 'ref')
      #p.join()
      #print x
      #print '\n'.join(out.split('\n')[1:]) # to remove first line of py3dna /tmp/xxx
      f.close()
      break
    align_all()
    rr()
    cmd.set('grid_mode', 1)


def mini(f):
    #os.system('/home/magnus/opt/qrnas/QRNA02/QRNA -i ' + f + ' -c /home/magnus/opt/qrnas/QRNA02/configfile.txt -o out.pdb')
    os.system('~/opt/qrnas/QRNA02/QRNA -i ' + f + ' -c ~/opt/qrnas/QRNA02/configfile.txt -o out.pdb')
    cmd.delete('mini')
    cmd.load('out.pdb', 'mini')
    print('end')


def reload():
    """Reload PyMOL4RNA.py"""
    cmd.run(RNA_TOOLS_PATH + "/rna_tools/tools/PyMOL4RNA/PyMOL4RNA.py")

def clr():
  """clr - make white bg and structure black"""
  cmd.bg_color( "white" )
  color_by_text('color black, all')


def rlabel():
    cmd = "n. C1'", '"%s %s" % (resn, resi)'
    print('label ' + cmd)
    cmd.label(cmd)


def sav(name):
    # cmd.bg_color( "white" )
    psefn = '~/Desktop/' + name + '.pse'
    cmd.save(psefn)
    coverfn = '/tmp/' + name + '.png'
    cmd.save(coverfn)
    os.system('/usr/local/bin/fileicon set ' + psefn + ' ' + coverfn)

    #cmd.png(coverfn, 576,576)
    #cmd.ray(576,576)
def hide_rna():
    cmd.hide('(polymer.nucleic)')
cmd.extend('rna-hide', hide_rna)

def show_rna():
    cmd.show('(polymer.nucleic)')
cmd.extend('rna-show', show_rna)

def select_rna():
    cmd.select('polymer.nucleic')
cmd.extend('select-rna', select_rna)

def hide_protein():
    cmd.hide('(polymer.protein)')
cmd.extend('protein-hide', hide_protein)
cmd.extend('rprotein-hide', hide_protein)

def select_protein():
    cmd.select('polymer.protein')
cmd.extend('protein-select', select_protein)

def tmp():
    cmd.save('/home/' + user + '/Desktop/' + tmp + '.png')
    cmd.save('/home/' + user + '/Desktop/' + tmp + '.pse')


################################################################################
def sav_tmp():
    from shutil import copyfile
    import datetime
    try:
        TMP_FOLDER + ' '
    except:
        print("Error: Set up TMP_FOLDER in your ~/.pymolrc, e.g. TMP_FOLDER = '/home/magnus/Desktop/PyMOL/'")
        return

    try:
        os.mkdir(TMP_FOLDER)
    except:
        pass

    date = datetime.datetime.today().strftime('%Y-%m-%d.%S')
    try:
        fn = TMP_FOLDER +  os.sep + id + '_' + date + '.pse'
    except TypeError:
        fn = TMP_FOLDER +  os.sep + '_' + date + '.pse'
    cmd.save(fn)
    print('Save...' + fn)
    cmd.save(fn.replace('.pse', '.png'))
    copyfile(fn, TMP_FOLDER + '/last.pse')

def load_tmp():
    print('Load...')
    cmd.load(TMP_FOLDER + '/last.pse')

def quickref():
    print('   PyMOL4RNA (rna-pdb-tools)  ')
    print('~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~')
    print('Quickref: ')
    print('  alter (sele), chain="B" ')
    print('  alter (sele), resv -= 4')
    print('  alter (chain B), resv -= 44 ')
    print("  select br. all within 15 of resi 574")
    print("  select br. all within 15 of resi 377 # O. ihheyensis")
    print('  select br. all within 15 of U6_snRNA and resi 80')
    print('  set dash_color, red; set dash_width, 4')
    print('  p - prepare seq for printing')
    print('  rp - rna present, object names only click to get compact legend')
    print('  rp17')
    print('  rna_cartoon')
    print('  rs')
    print('  rcomp')
    print('  color_obj')
    print('  color_rbw')
    print('  aa')
    print('  savt - save_transformed <object>, <file>')
    #print('  spl - color snRNAs of the spliceosome:'
    #    green: U2,  blue: U5, red:U6, orange:U2""")
    print('\_ RNA_TOOLS_PATH env variable used: ' + RNA_TOOLS_PATH)


try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
else:
    quickref()
    #cmd.set_key('CTRL-S', cmd.save, ['/home/magnus/Desktop/tmp.pse'])
    cmd.set_key('CTRL-S', sav_tmp)
    cmd.set_key('CTRL-Z', load_tmp)  # ostatni wrzucam tutaj
    #cmd.load, ['/home/magnus/Desktop/tmp.pse'])
    # main code #

    cmd.extend('quickref', quickref)
    cmd.extend('qr', quickref)
    cmd.extend('rp17', rp17)
    cmd.extend('rp17csrv', rp17csrv)
    cmd.extend('rg2', g2)
    cmd.extend('x', g2)

    cmd.extend('rp', rp)
    cmd.extend('p', p)
    cmd.extend('pdb', pdb)
    cmd.extend('seq', seq)
    cmd.extend('rseq', seq)
    cmd.extend('rna_cartoon', rna_cartoon)
    cmd.extend('rs', rs)
    cmd.extend('ino', ino)
    cmd.extend('rcomp', rcomp)
    cmd.extend('color_obj', color_obj)
    cmd.extend('color_rbw', color_rbw)
    cmd.extend('aa', align_all)
    cmd.extend('ss', ss)
    cmd.extend('ss_all', ss_all)
    cmd.extend('clarna', clarna)
    cmd.extend('x3dna', x3dna)
    cmd.extend("rgyration", rgyration)
    #cmd.extend("spl", spl)
    cmd.extend("spl2", spl2)
    cmd.extend('rlabel', 'rlabel')

    cmd.extend('grid_on', grid_on)
    cmd.extend('grid_off', grid_off)
    cmd.extend('reload', reload)

    cmd.extend('color_aa_types', color_aa_types)

    cmd.extend('names', names)

    # set dash lines
    cmd.set('dash_color', 'red')
    cmd.set('dash_width', 4)

    cmd.extend('sav', sav)
    cmd.extend('save_transformed', save_transformed)
    cmd.extend('savt', save_transformed)
    cmd.extend('show_all_at_once', show_all_at_once)

    cmd.set('ignore_case', 'off')
    #cmd.set('cartoon_ring_mode', '3')
    #cmd.set('cartoon_ring_finder', '2')
    #cmd.extend('spl_select', spl_select)
    #    print('ignore_case made off')
    print('\_ PYMOL4RNA loading .... [ok]')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function
import pandas as pd
import argparse

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('csv1', help="", default="")
    parser.add_argument('--sep1', help="default is ,; can be also '\t'", default=",")
    parser.add_argument('csv2', help="", default="")
    parser.add_argument('--sep2', help="default is ,; can be also '\t'", default=",")
    parser.add_argument('mergeon', help="merge on column", default="")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument('--output', help="output csv", default="merged.csv")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    df1 = pd.read_csv(args.csv1, delimiter=args.sep1)
    df2 = pd.read_csv(args.csv2, delimiter=args.sep2)
    print(df1)
    print(df2)

    merged = pd.merge(df1, df2, on=args.mergeon)
    print(merged)
    merged.to_csv(args.output, index=False)
    print('Saved to %s' % args.output)



#!/usr/bin/env python3
"""Quick and ugly fix to update auto readme."""

import subprocess

if __name__ == '__main__':
    start_tag = "usage: rna_pdb_toolsx.py [-h] [--version] [-r] [--renum-atoms]"
    end_tag = "ACCCGCAAGGCCGACGGC GCCGCCGCUGGUGCAAGUCCAGCCACGCUUCGGCGUGGGCGCUCAUGGGU"
    #
    doc = subprocess.Popen('python rna_pdb_toolsx.py -h', shell=True, stdout=subprocess.PIPE).stdout.read().decode()
    print(doc)
    fn = ''
    f = open('../README.md', 'r')
    copy_line_flag = True
    for l in f:
        if copy_line_flag:
            fn += l
        if l.startswith(start_tag):
            fn += doc + '\n'
            copy_line_flag = False
        if l.strip().startswith(end_tag):
            copy_line_flag = True

    f = open('../README.md', 'w')
    f.write(fn)
    f.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Obtain a list of interaction in an RNA molecule where "Interaction" is purely distance based (defined by --cuf-off). Later, you can use it to calculate distance based INF, dINF ;-).

Example::

    [mm] rna_calc_inf$ git:(master)$ ./rna_calc_dinf.py  test_output/1Y26.pdb
    X    13   X   14          bp   G   C                  WW_cis   1
    X    13   X   83          bp   G   C                  WW_cis   1
    X    13   X   82          bp   U   C                  WW_cis   1
    X    14   X   15          bp   C   G                  WW_cis   1
    X    14   X   83          bp   G   G                  WW_cis   1
    X    14   X   81          bp   G   G                  WW_cis   1
    X    14   X   82          bp   U   G                  WW_cis   1

use clarna_compare.py::

    [mm] rna_calc_inf$ ./rna_calc_dinf.py  test_output/1Y26.pdb > 1Y26.pdb.outCR
    [mm] rna_calc_inf$ clarna_compare.py -iref 1Y26.pdb.outCR -ichk 1Y26.pdb.outCR
    1Y26.pdb.outCR                               1Y26.pdb.outCR      1.000      0.000      1.000      1.000      1.000      1.000      1.000      1.000

You can use ``-d`` to get a list of all interacting bases, something like::

    draw_dists([(13, 14),(13, 83),...(82, 83)])

so you can plot all interacting bases:

.. image:: ../../rna_tools/tools/rna_calc_inf/doc/1y26_dinf.png

Mind, that draw_dists works on C2 atoms, that might be different from atoms detected with the program (e.g. different base atom could be detected to make an interaction).

.. image:: ../../rna_tools/tools/rna_calc_inf/doc/1y26_dinf2.png

"""
from __future__ import print_function
import Bio.PDB
import argparse

v = False # verbose
ATOMS = {'G' : "N9 C8 N7 C5 C6 O6 N1 C2 N2 N3 C4".split(),
         'A' : "N9 C8 N7 C5 C6 N6 N1 C2 N3 C4".split(),
         'U' : "N1 C2 O2 N3 C4 O4 C5 C6".split(),
         'C' : "N1 C2 O2 N3 C4 N4 C5 C6".split()}

class Pair:
    def __eq__(self, other):
        """Check if Pair is equal.
        """
        if (self.aid == other.aid and self.bid == other.bid and self.achain == other.achain and self.bchain == other.bchain) \
            or (self.aid == other.bid and self.bid == other.aid and self.achain == other.bchain and self.bchain == other.achain):
            return True


    def __init__(self, a, b, aname, bname, achain, bchain):
        """
        aid = A10
        """
        self.a = a
        self.b = b
        self.aname = aname
        self.bname = bname
        self.achain = achain  # A
        self.bchain = bchain  # A
        self.aid = achain + str(a)  # A10
        self.bid = bchain + str(b)  # A10

    def __repr__(self):
        # return ("%s    %i   %s   %i          bp %s %s                  WW_cis   1" % (self.achain, self.a, self.bchain, self.b, self.aname, self.bname))
        return ("%s    %i   %s   %i          bp G C                   WW_cis   1" % (self.achain, self.a, self.bchain, self.b))   #  , self.aname, self.bname))

    def get_indexes(self):
        """ get_dists([[62, 71], """
        return(self.a, self.b)

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('file', help="a PDB file")

    parser.add_argument("-d", "--draw-dists",
                        action="store_true", help="")

    parser.add_argument("-c", "--cut-off",
                        action="store_true", help="", default=5)

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    return parser


def infx(filename, draw_dists, cut_off):
    parser = Bio.PDB.PDBParser(QUIET=True) # QUIET=True avoids comments on errors in the pdb.

    structures = parser.get_structure('struc', filename)
    structure = structures[0] # 'structures' may contain several proteins in this case only one.

    atoms = Bio.PDB.Selection.unfold_entities(structure, 'A') # Atom
    ns = Bio.PDB.NeighborSearch(atoms)

    pairs = []

    # I'm working on A and B residues.
    for atom in atoms:
        if v: print(atom.get_parent().get_resname().strip(), atom.get_name())
        if atom.get_name() in ATOMS[atom.get_parent().get_resname().strip()]:  # '  C' so strip it
            if v: print('    Get distance')
            #target_atom = structure['A'][311]['CA']
            # Bs  = Bio.PDB.Selection.unfold_entities(structure, 'A')
            # print(atom.get_coord(), atom.get_parent())
            close_atoms = ns.search(atom.get_coord(), cut_off)
            for close_atom in close_atoms:

                bres = close_atom.get_parent()
                ares = atom.get_parent()

                if bres != ares:
                    a = bres.get_id()[1]  # to get res index
                    b = ares.get_id()[1]
                    # create an object Pair
                    p = Pair(b, a, bres.get_resname(), ares.get_resname(), bres.get_parent().id, ares.get_parent().id)
                    # and check if add it to the list?
                    if p not in pairs:
                        pairs.append(p)

        #print("A    %i   A   %i          bp C G                  X   1" % p)

    #print(close_atoms)
    #for a in close_atoms:
    #    print(a, a.get_parent())
    if not draw_dists:
        print('Classifier: Clarna')
        for p in pairs:
            print(p)

    if draw_dists:
        print('draw_dists([' + ','.join([str(p.get_indexes()) for p in pairs]) + '])')


# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    infx(args.file, args.draw_dists, args.cut_off)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
rna_calc_rmsd_multi_targets.py - calculate RMSDs of structures to multiple targets::

   $ rna_calc_rmsd_multi_targets.py --models multi-targets/rp21/*.pdb 
                                    --targets multi-targets/rp21/solutions/*.pdb 
                                    --target-selection A:1-27+29-41 
                                    --model-selection A:1-27+29-41

CSV table produced::

                              21_solution_0_ChainA.pdb  21_solution_0_ChainB.pdb  21_solution_1_ChainA.pdb  21_solution_1_ChainB.pdb  21_solution_2.pdb   mean    min    max    sd
    fn
    21_3dRNA_1_rpr.pdb                           12.17                     12.11                     12.17                     12.11              12.11  12.13  12.11  12.17  0.03
    21_Adamiak_1_rpr.pdb                          4.64                      4.61                      4.64                      4.61               4.64   4.63   4.61   4.64  0.01
    21_ChenHighLig_1_rpr.pdb                      4.01                      3.97                      4.01                      3.97               4.07   4.01   3.97   4.07  0.04
    21_Das_1_rpr.pdb                              5.71                      5.60                      5.71                      5.60               5.61   5.65   5.60   5.71  0.05
    --------------------------------------------------------------------------------
    Save rna_calc_rmsd_multi_targets_output.csv


"""
import argparse
import os
import pandas as pd


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    # parser.add_argument('-args', help="", default="")
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("--models", help="", nargs='+')
    parser.add_argument("--targets", help="", nargs='+')
    parser.add_argument("--output-csv", help="", default='rna_calc_rmsd_multi_targets_output.csv')
    parser.add_argument("--model-selection",
                         default='',
                         help="selection, e.g. A:10-16+20, where #16 residue is included")
    parser.add_argument("--target-selection",
                         default='',
                         help="selection, e.g. A:10-16+20, where #16 residue is included")
    
    return parser


def hr(t):
    print
    if t:
        print(t)
    print('-' * 80)


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()
    
    if args.verbose:
        print(args.models)
        print(args.targets)

    rmsds_files = []
    # run rmsd_calc_rmsd.py
    for target in args.targets:
        hr(target)
        outfile = os.path.basename(target).replace('.pdb', '.csv')
        cmd = "rna_calc_rmsd.py -t " + target + " " + ' '.join(args.models) + ' -o ' + outfile + ' --target-column-name '
        if args.target_selection:
            cmd += ' --target-selection ' + args.target_selection
        if args.model_selection:
            cmd += ' --model-selection ' + args.model_selection
        if args.verbose: print(cmd)
        os.system(cmd)
        rmsds_files.append(outfile)
    # process the results
    dfs = []
    for f in rmsds_files:
        df1 = pd.read_csv(f, index_col='fn')
        dfs.append(df1)

    df = pd.concat(dfs, axis=1)# join='inner')
    mean = df.mean(numeric_only=True, axis=1)
    median = df.median(numeric_only=True, axis=1)
    var = df.var(numeric_only=True, axis=1)
    min = df.min(numeric_only=True, axis=1)
    max = df.max(numeric_only=True, axis=1)
    sd = df.std(numeric_only=True, axis=1)

    df['mean'] = mean
    df['median'] = median
    df['variance'] = var
    df['min'] = min
    df['max'] = max
    df['sd'] = sd
    
    df.round(2).to_csv(args.output_csv)
    print('-' * 80)
    print(df.round(2))
    print('-' * 80)
    print('Save %s' % args.output_csv)



#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""rna_calc_rmsd.py - calculate RMSDs of structures to the target

If you still have problem with various number of atoms, check out this issue: get_rnapuzzle_ready: how to deal with `Alternate location indicator (https://github.com/mmagnus/rna-pdb-tools/issues/30).

The program is using (https://github.com/charnley/rmsd).

Example #1::

    $ rna_calc_rmsd.py -t 6_0_solution_4GXY_rpr.pdb --model-selection=A:1-17+24-110+115-168 *.pdb
    rmsd_calc_rmsd_to_target
    --------------------------------------------------------------------------------
    method: all-atom-built-in
    # of models: 35
    6_0_solution_4GXY_rpr.pdb 0.0 3409
    6_Blanchet_1_rpr.pdb 22.31 3409
    6_Blanchet_2_rpr.pdb 21.76 3409
    6_Blanchet_3_rpr.pdb 21.32 3409
    6_Blanchet_4_rpr.pdb 22.22 3409
    6_Blanchet_5_rpr.pdb 24.17 3409
    6_Blanchet_6_rpr.pdb 23.28 3409
    6_Blanchet_7_rpr.pdb 22.26 3409
    6_Bujnicki_1_rpr.pdb 36.95 3409
    6_Bujnicki_2_rpr.pdb 30.9 3409
    6_Bujnicki_3_rpr.pdb 32.1 3409
    6_Bujnicki_4_rpr.pdb 32.04 3409
    ...

Example #2::

    time rmsd_calc_to_target.py
      -t 5k7c_clean_onechain_renumber_as_puzzle_srr.pdb
      --target-selection A:1-48+52-63
      --model-selection A:1-48+52-63
      --target-ignore-selection A/57/O2\\'
      clusters/*_AA.pdb

    rmsd_calc_rmsd_to_target
    --------------------------------------------------------------------------------
      target_selection:  A:1-48+52-63
      model_selection:   A:1-48+52-63
      target_ignore_selection:  A/57/O2'
      model_ignore_selection:
    # of models: 801
    fn,rmsd_all
    pistol_thrs0.50A_clust01-000001_AA.pdb,7.596
    pistol_thrs0.50A_clust02-000001_AA.pdb,7.766
    pistol_thrs0.50A_clust03-000001_AA.pdb,18.171
    [..]
    pistol_thrs0.50A_clust799-000001_AA.pdb,5.356
    pistol_thrs0.50A_clust800-000001_AA.pdb,15.282
    pistol_thrs0.50A_clust801-000001_AA.pdb,16.339
    # of atoms used: 1237
    csv was created!  rmsds.csv
    rmsd_calc_to_target.py -t 5k7c_clean_onechain_renumber_as_puzzle_srr.pdb
    37.93s user 1.07s system 87% cpu 44.650 total

"""
from __future__ import print_function

from rna_tools.tools.rna_calc_rmsd.lib.rmsd.calculate_rmsd import *
import sys
from rna_tools.tools.extra_functions.select_fragment import select_pdb_fragment_pymol_style, select_pdb_fragment
import argparse
import sys
import math
import glob
import re
import os
import pandas as pd
pd.set_option('display.max_rows', 1000)

def get_rna_models_from_dir(files):
    """
    :param models: a list of filenames

    Example of the list::

       ['test_data/rp17/2_restr1_Michal1.pdb_clean.pdb', 'test_data/rp17/2a_nonrestr2_Michal1.pdb_clean.pdb',
       'test_data/rp17/3_nonrestr1_Michal1.pdb_clean.pdb', 'test_data/rp17/5_restr1_Michal3.pdb_clean.pdb']"""

    models = []
    #if not os.path.exists(directory):
    #    raise Exception('Dir does not exist! ', directory)
    #files = glob.glob(directory + "/*.pdb")
    files_sorted = sort_nicely(files)
    for f in files_sorted:
        models.append(f)
    return models

def sort_nicely( l ):
   """ Sort the given list in the way that humans expect.

   http://blog.codinghorror.com/sorting-for-humans-natural-sort-order/
   """
   convert = lambda text: int(text) if text.isdigit() else text
   alphanum_key = lambda key: [ convert(c) for c in re.split('([0-9]+)', key) ]
   l.sort( key=alphanum_key )
   return l

def calc_rmsd_pymol(pdb1, pdb2, method):
    """Calculate rmsd using PyMOL. Two methods are available: align and fit

    See:

    -  Align: http://www.pymolwiki.org/index.php/Align
    -  Fit:   http://www.pymolwiki.org/index.php/Fit

    Align can return a list with 7 items:

    RMSD after refinement
    Number of aligned atoms after refinement
    Number of refinement cycles
    RMSD before refinement
    Number of aligned atoms before refinement
    Raw alignment score
    Number of residues aligned

    in this version of function, the function returns `RMSD before refinement`.

    Install on OSX: ``brew install homebrew/science/pymol`` and set ``PYTHONPATH`` to
    your PyMOL packages, .e.g ::

      PYTHONPATH=$PYTHONPATH:/opt/local/Library/Frameworks/Python.framework/Versions/2.7/lib/python2.7/site-packages

    If problem::

      Match-Error: unable to open matrix file '/opt/local/Library/Frameworks/Python.framework/Versions/2.7/lib/python2.7/site-packages/data/pymol/matrices/BLOSUM62'.

    then define ``PYMOL_PATH`` in your .bashrc, e.g.::

       export PYMOL_PATH=/opt/local/Library/Frameworks/Python.framework/Versions/2.7/lib/python2.7/site-packages/pymol/
     """

    try:
        import __main__
        __main__.pymol_argv = ['pymol', '-qc']
        import pymol  # import cmd, finish_launching
        pymol.finish_launching()
    except ImportError:
        print('calc_rmsd_pymol: you need to have installed PyMOL')
        sys.exit(0) # no error

    pymol.cmd.reinitialize()
    pymol.cmd.delete('all')
    pymol.cmd.load(pdb1, 's1')
    pymol.cmd.load(pdb2, 's2')
    if method == 'align':
        # experiments with align <https://pymolwiki.org/index.php/Align>
        # quiet = 0/1: suppress output {default: 0 in command mode, 1 in API}
        return  (pymol.cmd.align('s1', 's2',quiet=1, object='aln')[3],0) #, pymol.cmd.align('s1','s2')[4])
        #raw_aln = pymol.cmd.get_raw_alignment('aln')
        #print raw_aln
        #for idx1, idx2 in raw_aln:
        #    print '%s`%d -> %s`%d' % tuple(idx1 + idx2)
        #pymol.cmd.save('aln.aln', 'aln')

    if method == 'fit':
        return (pymol.cmd.fit('s1', 's2'), 'fit')

def calc_rmsd(a,b, target_selection, target_ignore_selection, model_selection, model_ignore_selection, verbose):
    """
    a is model
    b is target

    :params: a = filename of structure a
    :params: b = filename of structure b

    :return: rmsd, number of atoms
    """
    if verbose: print('in:', a)
    atomsP, P = get_coordinates(a, model_selection, model_ignore_selection, 'pdb', True)
    atomsQ, Q = get_coordinates(b, target_selection,target_ignore_selection,  'pdb', True)

    if verbose:
        print(atomsP, P)
        print(atomsQ, Q)

    if atomsQ != atomsP:
        print('Error: # of atoms is not equal target (' + b + '):' + str(atomsQ) + ' vs model (' + a + '):' + str(atomsP))
        return (-1,0) # skip this RNA
    # Calculate 'dumb' RMSD
    normal_rmsd = rmsd(P, Q)
    # Create the centroid of P and Q which is the geometric center of a
    # N-dimensional region and translate P and Q onto that center.
    # http://en.wikipedia.org/wiki/Centroid
    Pc = centroid(P)
    Qc = centroid(Q)
    P -= Pc
    Q -= Qc

    if False:
        V = rotate(P, Q)
        V += Qc
        write_coordinates(atomsP, V)
        quit()

    return round(kabsch_rmsd(P, Q),2), atomsP

def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawTextHelpFormatter)#formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('-t',"--target-fn",
                            default='',
                         help="pdb file")

    parser.add_argument("--target-selection",
                            default='',
                         help="selection, e.g. A:10-16+20, where #16 residue is included")

    parser.add_argument("--target-ignore-selection",
                            default='',
                         help="A/10/O2\'")

    parser.add_argument("--model-selection",
                            default='',
                         help="selection, e.g. A:10-16+20, where #16 residue is included")

    parser.add_argument("--model-ignore-selection",
                            default='',
                         help="A/10/O2\'")

    parser.add_argument('-m', "--method",
                         default='all-atom-built-in',
                         help="align, fit")

    parser.add_argument('-o', "--rmsds-fn",
                         default='rmsds.csv',
                         help="ouput, matrix")

    parser.add_argument("-v", "--verbose", action="store_true",
                        help="verbose")

    parser.add_argument('-pr', '--print-results',
                         action="store_true")

    parser.add_argument('-sr', '--sort-results',
                         action="store_true")

    parser.add_argument('-pp', '--print-progress',
                         default=False,
                         action="store_true")

    parser.add_argument("--target-column-name", action="store_true",
                        help="")

    parser.add_argument('files', help='files', nargs='+')

    return parser
# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    input_files = args.files  # opts.input_dir
    rmsds_fn = args.rmsds_fn
    target_fn = args.target_fn
    method = args.method

    print('# method:', method)

    target_selection = select_pdb_fragment(args.target_selection)
    model_selection = select_pdb_fragment(args.model_selection)

    if target_selection:
        if args.verbose:
            print('  target_selection: #', args.target_selection, target_selection)
            ts = target_selection
            print(ts)
            resides_in_total = 0
            for i in target_selection:
                print((i, len(ts[i]))) # chain string, a list of residues
                resides_in_total += len(ts[i])
            print('in total:', resides_in_total)

    if model_selection:
        if args.verbose:
            print('  model_selection:  ', len(model_selection), args.model_selection, model_selection)
            resides_in_total = 0
            for i in model_selection:
                print(i, len(model_selection[i])) # chain string, a list of residues
                resides_in_total += len(model_selection[i])
            print('in total:', resides_in_total)

    if args.target_ignore_selection:
        target_ignore_selection = select_pdb_fragment_pymol_style(args.target_ignore_selection, args.verbose)
    else:
        target_ignore_selection = None

    if args.model_ignore_selection:
        model_ignore_selection = select_pdb_fragment_pymol_style(args.model_ignore_selection, args.verbose)
    else:
        model_ignore_selection = None

    if  args.target_ignore_selection:
        if args.verbose: print('  target_ignore_selection: ', args.target_ignore_selection)
    if  args.model_ignore_selection:
        if args.verbose: print('  model_ignore_selection:  ', args.model_ignore_selection)

    models = get_rna_models_from_dir(input_files)

    print('# of models:', len(models))

    f = open(rmsds_fn, 'w')
    #t = 'target:' + os.path.basename(target_fn) + ' , rmsd_all\n'
    if args.target_column_name:
        t = 'fn,' + os.path.basename(args.target_fn) + '\n'
    else:
        t = 'fn,rmsd_all\n'

    c = 1
    for r1 in models:
        if method == 'align' or method == 'fit':
            rmsd_curr, atoms = calc_rmsd_pymol(r1, target_fn, method, args.verbose)
        else:
            rmsd_curr, atoms = calc_rmsd(r1, target_fn, target_selection, target_ignore_selection, model_selection, model_ignore_selection, args.verbose)
        r1_basename = os.path.basename(r1)
        if args.print_progress: print(r1_basename, rmsd_curr, atoms)
        t += r1_basename + ',' + str(round(rmsd_curr,3)) + ' '
        c += 1
        t += '\n'

    f.write(t)
    f.close()

    #print t.strip() # matrix

    print('# of atoms used:', atoms)

    df = pd.read_csv(rmsds_fn)
    df = df.round(2)
    if args.sort_results:
        df = df.sort_values('rmsd_all', ascending=True)
    if args.print_results:
        print(df)
    df.to_csv(rmsds_fn, sep=',', index=False)  # easy to set \t here!

    print('csv was created! ', rmsds_fn)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Test unit for lib.format"""

import os
import sys
import shutil
import uuid
from tempfile import tempdir
from unittest import TestCase, main

__file__ == 'test.py'

DIRNAME = os.path.dirname(__file__)
TEST_LINE = 'ATOM     14  C4\'   U A   5      25.964   2.534  -0.000  1.00  0.0               '

sys.path.append(os.path.abspath(os.path.join(DIRNAME, os.path.pardir)))
import SingleLineUtils
from PDBFile import PDBFile


class FormatlibTests(TestCase):
    """ Tests for formatlib"""
    def set_up(self):
        """Formatlib: Sets up test suite."""
        pass

    # SingleLineUtils tests
    def test_get_atom_code(self):
        result = SingleLineUtils.get_atom_code(TEST_LINE)
        self.assertEqual(result, 'C4\'')

    def test_get_atom_coords(self):
        result = SingleLineUtils.get_atom_coords(TEST_LINE)
        self.assertEqual(result, (5.964, 2.534, -0.0))

    def test_get_res_code(self):
        result = SingleLineUtils.get_res_code(TEST_LINE)
        self.assertEqual(result, '  U')

    def test_get_res_num(self):
        result = SingleLineUtils.get_res_num(TEST_LINE)
        self.assertEqual(result, 5)

    def test_set_line_bfactor(self):
        result = SingleLineUtils.set_line_bfactor(TEST_LINE, 10.5)
        correct = 'ATOM     14  C4\'   U A   5      25.964   2.534  -0.000  1.00 10.50              '
        self.assertEqual(result, correct)

    def test_set_atom_code(self):
        result = SingleLineUtils.set_atom_code(TEST_LINE, 'Xx')
        correct = 'ATOM     14  Xx    U A   5      25.964   2.534  -0.000  1.00  0.0               '
        self.assertEqual(result, correct)

    def test_set_res_code(self):
        result = SingleLineUtils.set_res_code(TEST_LINE, 'Xx')
        correct = 'ATOM     14  C4\'   XxA   5      25.964   2.534  -0.000  1.00  0.0               '
        self.assertEqual(result, correct)

    # tests for PDBFile
    def test_validate_pdb_true(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test0.in'))
        self.assertTrue(pdb_file.validate_pdb())

    def test_validate_pdb_false(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test.py'))
        self.assertFalse(pdb_file.validate_pdb())

    def test_detect_proteins(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'protein.pdb'))
        self.assertTrue(pdb_file.detect_proteins())

    def test_seq_from_pdb(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test0.in'))
        self.assertEqual(pdb_file.seq_from_pdb(), 'GAGCCGUAUGCGAUGAAAGUCGCACGUACGGUUC')

    def test_get_fasta(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test0.in'))
        self.assertEqual(pdb_file.get_fasta(name='test'), '>test\nGAGCCGUAUGCGAUGAAAGUCGCACGUACGGUUC\n')

    def test_remove_non_atoms(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test_nmr.in'))
        pdb_file.remove_non_atoms()
        with open(os.path.join(DIRNAME, 'test_nmr_atom.out')) as f:
            correct = f.read()
        self.assertEqual(pdb_file.pdb_string, correct)

    def test_resname_check_and_1to3(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test0.in'))
        pdb_file.resname_check_and_3to1()
        with open(os.path.join(DIRNAME, 'test0_3to1.out')) as f:
            correct = f.read()
        self.assertEqual(pdb_file.pdb_string, correct)

    def test_terminate_chains(self):
        # TODO: write it
        pass

    def test_remove_short_chains(self):
        # TODO: write it
        pass

    def test_check_and_add_P_at_start(self):
        # TODO: write it
        pass

    def test_set_residues_bfactor(self):
        # TODO: write it
        pass

    def test_count_models(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test_nmr.in'))
        self.assertEqual(pdb_file.count_models(), 10)

    def test_check_and_get_first_model(self):
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test_nmr.in'))
        pdb_file.check_and_get_first_model()
        with open(os.path.join(DIRNAME, 'test_nmr.out')) as f:
            correct = f.read()
        self.assertEqual(pdb_file.pdb_string, correct)

    # global tests
    def test_0(self):
        """Formatlib: first test"""
        tmpfile = os.path.join(tempdir or '/tmp/', str(uuid.uuid4()))
        shutil.copyfile(os.path.join(DIRNAME, 'test0.in'), tmpfile)
        pdb_file = PDBFile(pdb_path=tmpfile)
        pdb_file.pedantic_pdb()
        result = pdb_file.pdb_string
        os.remove(tmpfile)
        f = open(os.path.join(DIRNAME, 'test0.out'))
        correct = f.read()
        f.close()
        self.assertEqual(correct, result)

    def test_1(self):
        """Formatlib: second test"""
        pdb_file = PDBFile(pdb_path=os.path.join(DIRNAME, 'test1.in'))
        pdb_file.pedantic_pdb()
        result = pdb_file.pdb_string
        f = open(os.path.join(DIRNAME, 'test1.out'))
        correct = f.read()
        f.close()
        self.assertEqual(correct, result)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_rosetta_check_progress.py - check progress for many simulations of Rosetta

Example::

    [peyote2] rosetta_jobs rna_rosetta_check_progress.py .
             jobs  #curr  #todo  #decoys done
    0  ./rp17s223    200      0      407  [ ]
    1   ./rp17hcf      0      0        0  [ ]
    # curr  232 #todo  0

"""
from __future__ import print_function
import subprocess

import os
import pandas as pd
import glob
import sys
import argparse

try:
    from rna_tools.rna_tools_config import RNA_ROSETTA_RUN_ROOT_DIR_MODELING, RNA_ROSETTA_NSTRUC
except:
    print('Set up RNA_ROSETTA_RUN_ROOT_DIR_MODELING in rna_tools.rna_tools_config_local.py')

from rna_tools.rna_tools_config import EASY_CAT_PATH
# print('Set up EASY_CAT_PATH in rpt_config_local.py')
print(EASY_CAT_PATH)


def run_cmd(cmd):
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('dir', default=RNA_ROSETTA_RUN_ROOT_DIR_MODELING,
                        help="""directory with rosetta runs, define by RNA_ROSETTA_RUN_ROOT_DIR_MODELING
right now: \n""" + RNA_ROSETTA_RUN_ROOT_DIR_MODELING)
    parser.add_argument('-v', '--verbose',
                        action='store_true', help="be verbose")
    parser.add_argument('-m', '--min-only', action='store_true',
                        help="check only for mo folder")
    parser.add_argument('-s', '--select',
                        help="select for analysis only jobs with this phrase, .e.g., evoseq_", default='')

    parser.add_argument('-k', '--kill', action='store_true', help="""kill (qdel) jobs if your reach
limit (nstruc) of structure that you want, right now is %i structures""" % RNA_ROSETTA_NSTRUC)
    return parser


if __name__ == '__main__':
    args = get_parser().parse_args()
    v = args.verbose

    # jobs = [x.replace('.fa','') for x in glob.glob( '*.fa')] # sys.argv[1] +
    # sys.argv[1] +
    # print jobs

    jobs = [x for x in glob.glob(args.dir + '/*')]
    if v:
        print(jobs)

    curr_path = os.getcwd()

    d = {}
    d['#decoys'] = []
    d['jobs'] = []
    d['#curr'] = []
    d['#todo'] = []
    d['done'] = []
    d['progress'] = []

    for j in jobs:
        if args.select:
            if not args.select in j:
                continue
        if v:
            print(j)
        d['jobs'].append(j)
        try:
            os.chdir(j)
        except:
            pass  # OSError: [Errno 20] Not a directory:

        dirs = []

        if not args.min_only:
            # Uff.. this is crazy.
            for c in ['_', '__', '___', 'x', 'y', 'z', 'X', 'Y', 'Z', 'o', 'out'] + ['r' + str(i) for i in range(0, 1000)]:
                if os.path.exists(c):
                    dirs.append(c)
            # print ' ', j, '', dirs
            cmd = EASY_CAT_PATH + ' ' + ' '.join(dirs)
            if v:
                print('cmd: %s' % cmd)

            out, err = run_cmd(cmd)

            if out.strip():
                if len(j) <= 5:
                    if v:
                        print(out + '\t\t', end='')
                else:
                    if v:
                        print(out + '\t', end='')
                no_decoys = int(out.split()[-2])
            else:
                no_decoys = 0

            if v:
                print('#', no_decoys)
        else:
            # # of minimized
            RNA_ROSETTA_NSTRUC = 1600  # change expected !!!! pretty ugly
            for c in ['mo']:
                if os.path.exists(c):
                    dirs.append(c)
            # print ' ', j, '', dirs
            cmd = EASY_CAT_PATH + ' ' + ' '.join(dirs)
            if v:
                print(cmd)

            out, err = run_cmd(cmd)

            if out.strip():
                if len(j) <= 5:
                    if v:
                        print(out + '\t\t', end='')
                else:
                    if v:
                        print(out + '\t', end='')
                no_decoys = int(out.split()[-2])
            else:
                no_decoys = 0

        if v:
            print('#', no_decoys)

        # check current running, only 6 char are taken from dir name
        # /home/magnus/rosetta_jobs/rp17s223 -> rp17s2199, rp17s2185
        cmd = 'qstat | grep ' + \
            os.path.basename(j)[:6] + ' | grep "  r  " | wc -l '
        if v:
            print(cmd)

        out, err = run_cmd(cmd)

        if v:
            print('@cluster #curr ', out, end="")
        curr = int(out)
        d['#curr'].append(curr)

        # check todo
        out, err = run_cmd('qstat | grep ' + os.path.basename(j)
                           [:6] + ' | grep "  qw  " | wc -l ')
        if v:
            print('#todo ', out, end="")

        todo = int(out)
        d['#todo'].append(todo)

        d['#decoys'].append(no_decoys)

        if no_decoys >= RNA_ROSETTA_NSTRUC:
            if v:
                print('# @cluster:', out, ' < ............... OK')
            d['done'].append('[x]')
            if args.kill:
                cmd = 'qstat | grep ' + \
                    os.path.basename(j)[:6] + \
                    " | awk '{print $1}' | xargs qdel "
                print(cmd)
                os.system(cmd)
        else:
            d['done'].append('[ ]')
            if v:
                print('# @cluster:', out)
            # cmd = 'kill '
            # print cmd

        d['progress'].append(round((float(no_decoys) / 10000) * 100, 2))

        os.chdir(curr_path)

    df = pd.DataFrame(d, columns=['jobs', '#curr', '#todo', '#decoys', 'done', 'progress'])
    print(df)

    out, err = run_cmd('qstat | grep magnus  | grep "  r  " | wc -l ')
    print('#curr ', out, end=" ")
    out, err = run_cmd('qstat | grep magnus | grep "  qw  " | wc -l ')
    print('#todo ', out)



#!/usr/bin/env python

from Bio import PDB
from Bio.PDB import PDBIO
from Bio.PDB.Atom import PDBConstructionWarning

import warnings
warnings.simplefilter('ignore', PDBConstructionWarning)
import string
import re
import sys
import copy


class Struc:
    def __init__(self, ring, pdb, fragments, pdb_out, v):
        self.fragments = fragments
        self.ring = PDB.PDBParser().get_structure('ring', ring)
        self.pdb = PDB.PDBParser().get_structure('pdb', pdb)
        self.pdb_out = pdb_out
        self.v = v

    def detect_what_is_in_pdb(self):
        fragments = ''
        for c in self.pdb[0]:
            fragments += ';' + c.get_id() + ':'
            for r in c:
                fragments += str(r.get_id()[1]) + ','
            fragments = fragments[:-1]
        fragments = fragments[1:]  # to remoev first ;

        self.fragments = fragments
        print('fragments in pdb', self.fragments)

    def merge(self):
        v = self.v
        txt = self.fragments
        txt = txt.replace(' ', '')
        if v:
            print(txt)
        l = re.split('[,:;]', txt)
        if v:
            print(l)

        if v:
            print('ring', self.ring)

        for i in l:  # ['A', '1-10', '15', '25-30', 'B', '1-10']
            if i in string.ascii_letters:
                if v:
                    print('chain', i)
                chain_curr = i
                continue

            if i.find('-') > -1:
                start, ends = i.split('-')
                if start > ends:
                    return 'Error: range start > end ' + i
                index = list(range(int(start), int(ends) + 1))
            else:
                index = [int(i)]

            for i in index:
                try:
                    residue_ring = self.ring[0][chain_curr][i]
                except KeyError:
                    if i == chain_curr:
                        return 'Error: Chain ' + chain_curr + ' not found in the PDB structure (seq)'
                    else:
                        return 'Error: Residue ' + chain_curr + ':' + str(i) + ' found in the PDB structure (seq)'
                try:
                    residue_pdb = self.pdb[0][chain_curr][i]
                except KeyError:
                    if i == chain_curr:
                        return 'Error: Chain ' + chain_curr + ' not found in the PDB structure (seq)'
                    else:
                        return 'Error: Residue ' + chain_curr + ':' + str(i) + ' found in the PDB structure (seq)'

                if residue_ring.get_resname() == residue_pdb.get_resname():
                    if v:
                        print(i, residue_ring.get_resname(), residue_pdb.get_resname(), ' ...coping')
                    to_remove = self.ring[0][chain_curr][i].copy()
                    for a in to_remove:
                        self.ring[0][chain_curr][i].detach_child(a.get_id())  # remove atoms

                    for a in self.pdb[0][chain_curr][i]:
                        try:
                            self.ring[0][chain_curr][i].add(a)
                        except PDB.PDBExceptions.PDBConstructionException:
                            return 'Error: duplicated atoms in ' + chain_curr + ':' + str(i)
                else:
                    return 'Error: Residue ' + chain_curr + ':' + str(i) + ' not the same as in the sequence'

        io = PDBIO()
        io.set_structure(self.ring)
        io.save(self.pdb_out)
        print('Save as ', self.pdb_out)
        return False


if __name__ == '__main__':
    s = Struc(ring='test_data/1xjr_simrna_nstep_1.pdb',
              pdb='test_data/rna_sq_renumber.pdb', fragments='', pdb_out='tmp.pdb', v=True)
    s.detect_what_is_in_pdb()

    s = Struc(ring='test_data/1xjr_simrna_nstep_1.pdb', pdb='test_data/1xjr_root.pdb',
              fragments='A:1-14,37-45', pdb_out='test_data/tmp.pdb', v=True)
    err = s.detect_what_is_in_pdb()
    err = s.merge()
    if err:
        print(err)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_cal_rmsd_trafl_plot - generate a plot based of <rmsd.txt> of rna_calc_evo_rmsd.py."""

from __future__ import division

import pandas as pd
import matplotlib.pyplot as plt
import argparse
import numpy as np
import matplotlib.pyplot as plt
from pandas import Series, DataFrame

plt.style.use('ggplot')
plt.rc('figure', figsize=(10, 6))
np.set_printoptions(precision=4, threshold=500)
pd.options.display.max_rows = 100


def get_parser():
    """Get parser."""
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="rmsd.txt")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    fn = args.file
    df = pd.read_csv(fn, sep="\t")
    print(df.head())
    print(len(df))  # print len(df) #

    # plt.style.use('classic')
    ax = df.plot(x=df.columns[0], y=df.columns[1],
                 legend=False,
                 marker="o",
                 clip_on=False)  # (x='fn', y='rmsd')
    ax.set_ylabel(df.columns[1])
    plt.ylim(-0, df[df.columns[1]].max() + 1)
    plt.xlim(-0, df[df.columns[0]].max() + 1)

    outfn = args.file.replace('.txt', '') + '.png'
    print('Save plot %s' % outfn)
    plt.savefig(outfn)



#!/usr/bin/env python
"""A super simple script to get some statistics of who is running at a cluster

Set MAX_JOBS to calc % of usage, it's an approximation of max number of jobs, e.g. peyote ~1k (rather 700, e.g. FARNA runs.).

.. warning MAX_JOBS in hardcoded in the code. To fix at some point."""
from __future__ import print_function
import subprocess
import logging
import argparse

from rna_tools.rna_tools_config import CPUS_CLUSTER

MAX_JOBS = CPUS_CLUSTER
print('MAX_JOBS:', MAX_JOBS)

logger = logging.getLogger()
handler = logging.StreamHandler()
logger.addHandler(handler)
formatter = logging.Formatter("[ %(filename)s:%(lineno)s - %(funcName)20s() %(message)s]")
handler.setFormatter(formatter)
logger.addHandler(handler)


def stats_for_cluster():
    """get stats (#jobs) per cluster"""

    cmd = "/home/oge/bin/lx24-amd64/qstat -u '*'"

    logger.info(cmd)

    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = p.stdout.read().strip()

    cc = 0
    for l in out.split('\n'):
        if l.strip():
            if l.find('hqw') > -1:
                continue
            c = l.split()[-1]
            try:
                cc += int(c)
            except ValueError:
                pass
    return cc


def stats_for_user():
    """get stats (#jobs) per user"""
    cmd = "/home/oge/bin/lx24-amd64/qstat "  # -u '*'"

    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = p.stdout.read().strip()

    cc = 0
    for l in out.split('\n'):
        if l:
            c = l.split()[-1]
            try:
                cc += int(c)
            except ValueError:
                pass
    return cc


def per_user():
    """get stats (#cpus) per user"""
    # {'deepak': 160, 'azyla': 8, 'magnus': 755}
    cmd = "/home/oge/bin/lx24-amd64/qstat -u '*'"

    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = p.stdout.read().strip()

    per_user = {}
    for l in out.split('\n'):
        if l.startswith('job-ID') or l.startswith('---'):
            pass
        else:
            ll = l.split()
            user = ll[3]
            cpus = int(ll[-1])
            if user in per_user:
                per_user[user] += cpus
            else:
                per_user[user] = cpus
    return per_user


def get_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", help="increase output verbosity",
                        action="store_true")
    return parser


# main
if __name__ == '__main__':
    args = get_parser().parse_args()

    if args.verbose:
        logger.setLevel(logging.INFO)

    cc = stats_for_cluster()
    print('#jobs cluster', cc, 'load: ', cc / float(MAX_JOBS), ' to use:', MAX_JOBS - cc)
    cc = stats_for_user()
    print('#jobs you    ', cc, 'load: ', cc / float(MAX_JOBS), ' to use:', MAX_JOBS - cc)
    print(per_user())



#!/usr/bin/env python
"""rna_simrnaweb_download_job.py - download model files, trajectory for a given SimRNAweb job.

Usage::

    rp17pk$ rna_pdb_download_simrna_job.py 27b5093d -m -t -x
    # download more clusters, trajectory, extract100

    cp771_pk$ rna_pdb_download_simrna_job.py -t -x -m cf8f8bb2 -p cp771_pk
    # download with a trajectory, and cluster #4 and #5, add to all pdb files
    # prefix: cp771_pk

    $ rna_simrnaweb_download_job.py --web-models rp17_well_d10_e1-a43d3ab5 --prefix tar
    # prefix added will be tar_XXXX

Example::

    rna_pdb_download_simrna_job.py -t -x -m 20569fa1 -p zmp_pk

    [mm] zmp_pk ls
    20569fa1_ALL_100low.trafl
    _20569fa1-thrs7.10A_clust04
    _20569fa1-thrs7.10A_clust05
    _20569fa1_ALL_100low
    data
    rna_simrna_extract.log
    subset.png
    zmp_pk_20569fa1-thrs7.10A_clust01X.pdb
    zmp_pk_20569fa1-thrs7.10A_clust02X.pdb
    zmp_pk_20569fa1-thrs7.10A_clust03X.pdb
    zmp_pk_20569fa1-thrs7.10A_clust04X.pdb
    zmp_pk_20569fa1-thrs7.10A_clust05X.pdb

.. downloaded clusters from 1 to 5, all pdb files have added prefix `zmp_pk`.
"""

# Ideas:  71707ff4-fe16-4b78-8340-78913312a547_ALL_thrs12.50A_clust01-000001_AA keep
#                                              //


import argparse
import os
import urllib3
import shutil
import subprocess
import sys
import logging
import time

sys.tracebacklimit = 0

SIMRNAWEB_ARCHIVE = "/home/magnus/work/simrnaweb-archive"

class SimRNAwebError(Exception):
    pass


def extract(job_id, nstruc, remove_trajectory, lowest=True):
    """using rna_simrna_extract.py"""
    if lowest:
        # Ok, this is off for now because I can download top100 and top200
        cmd = 'rna_simrna_lowest.py -n ' + str(nstruc) + ' ' + job_id + '*.trafl'
        print(cmd)
        os.system(cmd)
    if remove_trajectory:
        os.system('rm *_ALL.trafl*')
    cmd = 'rna_simrna_extract.py -t ' + job_id + '*01X.pdb -f *' + job_id + '*top' + str(nstruc) + '.trafl -n ' + str(nstruc) + ' -c'
    print(cmd)
    os.system(cmd)


def download_trajectory():
    cmd = "wget --restrict-file-names=nocontrol http://genesilico.pl/SimRNAweb/media/jobs/" + job_id + \
        "/processing_results/" + job_id + "_ALL.trafl "  # -O " + fn
    print(cmd)
    os.system(cmd)

def download_trajectory_top100():
    cmd = "wget --restrict-file-names=nocontrol http://genesilico.pl/SimRNAweb/media/jobs/" + job_id + \
        "/processing_results/top100/" + job_id + "_ALL_top100.trafl "  # -O " + fn
    print(cmd)
    os.system(cmd)

def download_trajectory_top200():
    cmd = "wget --restrict-file-names=nocontrol http://genesilico.pl/SimRNAweb/media/jobs/" + job_id + \
        "/processing_results/top200/" + job_id + "_ALL_top200.trafl "  # -O " + fn
    print(cmd)
    os.system(cmd)

def run_cmd(cmd):
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err

def copy_trajectory(args):
    """Using xfind https://github.com/mmagnus/Xfind

    Do I have to copy it? Nope, you can make a link (ln)"""
    # OK, find this file on the drive
    # cmd = "mdfind -name " +  args.job_id  + " 000001_AA.pdb"
    print('[Get Trajectory]')
    cmd = "mdfind -name " + args.job_id + " | grep .ALL.trafl"
    print(cmd)
    out, err = run_cmd(cmd)
    if out:
        path_to_fn = out.split('\n')[0]
        cmd = 'ln -s ' + path_to_fn + " " + job_id + "_ALL.trafl "
        print(cmd)
        os.system(cmd)
        return

    clustername = "malibu"
    cmd = "ssh " + clustername + " xfind " + args.job_id + " | grep .ALL.trafl"
    print(cmd)
    out, err = run_cmd(cmd)
    if out:
        path_to_trafl = out.split('\n')[0]
        os.system('scp ' + clustername + ':' + path_to_trafl + " " + SIMRNAWEB_ARCHIVE + '/' + job_id + "_ALL.trafl ")
        return

    raise Exception("Trajectory not found on the cluster")

def copy_template(args):
    """Copy a template for the trajectory, on cluster and locally.

    Args:
      args.job_id

    If it fails, raise an Exception"""
    # OK, find this file on the drive
    # cmd = "mdfind -name " +  args.job_id  + " 000001_AA.pdb"
    cmd = "mdfind -name " +  args.job_id  + " | grep 'clust01X.pdb$'"  #'000001_AA.pdb$'"
    print(cmd)
    out, err = run_cmd(cmd)
    if out:
        path_to_fn = out.split('\n')[0]
        print('Template found %s' % path_to_fn)
        # cmd = 'cp ' + path_to_fn + " " + job_id + "-01X.pdb"
        shutil.copyfile(path_to_fn, job_id + "-01X.pdb")
        return

    cmd = "mdfind -name " +  args.job_id  + " | grep '01X.pdb$'"
    print(cmd)
    out, err = run_cmd(cmd)
    if out:
        path_to_fn = out.split('\n')[0]
        print('Template found %s' % path_to_fn)
        # cmd = 'cp ' + path_to_fn + " " + job_id + "-01X.pdb"
        shutil.copyfile(path_to_fn, job_id + "-01X.pdb")
        return

    cmd = "mdfind " +  args.job_id  + " | grep '000001_AA.pdb$'"
    print(cmd)
    out, err = run_cmd(cmd)
    if out:
        path_to_fn = out.split('\n')[0]
        print('Template found %s' % path_to_fn)
        # cmd = 'cp ' + path_to_fn + " " + job_id + "-01X.pdb"
        shutil.copyfile(path_to_fn, job_id + "-01X.pdb")
        return

    # find it on the cluster
    clustername = "malibu"
    cmd = "ssh " + clustername + " xfind " + args.job_id + " | grep 'clust01-000001.pdb$'"
    print(cmd)
    out, err = run_cmd(cmd)
    if out:
        path_to_fn = out.split('\n')[0]
        print('Template found %s' % path_to_fn)
        cmd = 'scp ' + clustername + ':' + path_to_fn + " " + job_id + "-01X.pdb"
        print(cmd)
        os.system(cmd)
        return

    # hack with returns, not pretty
    raise Exception("Trajectory template not found on the cluster")

def add_prefix(prefix):
    """This is very crude way to do it"""
    # d2b57aef_ALL-thrs8.40A_clust01X.pdb -> gba_pk_d2b57aef_ALL-thrs8.40A_clust01X.pdb
    # print('disable right now')
    os.system("rename 's/^/" + prefix.strip() + "_/' *pdb")
    # os.system("rename 's/^/" + args.prefix.strip() + "_/' *_ALL.tarfl*")


def more_clusters(args):
    print('more clusters')
    http = urllib3.PoolManager()
    url = "http://genesilico.pl/SimRNAweb/media/jobs/" + job_id + "/processing_results/"
    response = http.request('GET', url)
    if not response.status == 200:
        raise SimRNAwebError('Job not found on the server: %s' % job_id)
    html = response.data

    for l in html.split('\n'):
        if 'clust04.trafl' in l or 'clust05.trafl' in l:
            fn = l.split('"')[1]
            print(fn)

            # shorten names
            nfn = fn.replace("-000001", '').replace('_AA',
                                                    'X').replace('_ALL_', '-')
            parts = nfn.split('-')
            print(parts)
            # nfn = '-'.join([fn[:12], ''.join(parts[-1])])

            # wget
            cmd = "wget http://genesilico.pl/SimRNAweb/media/jobs/" + \
                job_id + "/processing_results/" + fn + " -O " + nfn
            os.system(cmd)

            if 'clust04.trafl' in l:
                cmd = 'rna_simrna_extract.py -t *01X.pdb -f *04.trafl -c -n 1'
                subprocess.check_call(cmd, shell=True)
                os.remove(nfn)
                shutil.move(nfn.replace('.trafl', '-000001_AA.pdb'),
                            nfn.replace('.trafl', 'X.pdb'))
            if 'clust05.trafl' in l:
                cmd = 'rna_simrna_extract.py -t *01X.pdb -f *05.trafl -c -n 1'
                subprocess.check_call(cmd, shell=True)
                os.remove(nfn)
                # 27b5093d-thrs6.20A_clust05-000001_AA.pdb -> 27b5093d-thrs6.20A_clust05X.pdb
                shutil.move(nfn.replace('.trafl', '-000001_AA.pdb'),
                            nfn.replace('.trafl', 'X.pdb'))


def download_models(args):
    # download models, get propare names of pdb files
    # http://genesilico.pl/SimRNAweb/media/jobs/d86c07d9-9871-4454-bfc6-fb2e6edf13fc/output_PDBS/d86c07d9-9871-4454-bfc6-fb2e6edf13fc_ALL_thrs12.50A_clust01-000001_AA.pdb
    http = urllib3.PoolManager()
    url = "http://genesilico.pl/SimRNAweb/media/jobs/" + \
        job_id + "/output_PDBS/"
    response = http.request('GET', url)
    if not response.status == 200:
        raise SimRNAwebError('Job not found on the server: %s' % job_id)
    html = response.data

    for l in html.split('\n'):
        if l.find('AA.pdb') > -1 and l.find('clust') > -1:
            # find fn
            fn = l.split('"')[1]

            # shorten names
            nfn = fn.replace("-000001", '').replace('_AA',
                                                    'X').replace('_ALL_', '-')
            parts = nfn.split('-')
            print(parts)
            # nfn = '-'.join([fn[:12], ''.join(parts[-1])]) ### ? nfn

            # wget
            cmd = "wget --restrict-file-names=nocontrol http://genesilico.pl/SimRNAweb/media/jobs/" + job_id + "/output_PDBS/" + \
                  fn + " -O " + nfn.replace('%2B', '+')  # fix
            print(cmd)
            subprocess.check_call(cmd, shell=True)


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('job_id', help='job_id')
    parser.add_argument(
        '-p', '--prefix', help='prefix to the name, withouth _, be careful with this.'
        'If you have already some files with the given folder, their names might'
        'be changed.')
    parser.add_argument('-n', '--nstruc',
                       help='extract nstruc the lowest energy, this option must go with --web', type=int, default=100)
    parser.add_argument('-e', '--extract',
                       help='extract nstruc the lowest energy, this option must go with --web', action="store_true")
    ## parser.add_argument('-t', '--trajectory',
    ##                     action='store_true', help='download also trajectory')
    parser.add_argument('-m', '--more_clusters',
                        action='store_true', help='download also cluster 4 and 5')
    parser.add_argument('-r', '--remove-trajectory',
                        action='store_true', help='remove trajectory after analysis', default=False)
    parser.add_argument('-c', '--cluster',
                        action='store_true', help='get trajectory from cluster OR local on your computer (mdfind for macOS)', default=False)
    parser.add_argument('-d', '--download-trajectory',
                        action='store_true', help='web', default=False)
    parser.add_argument('--top100', action='store_true', help='download top100 trajectory', default=False)
    parser.add_argument('--top200', action='store_true', help='download top200 trajectory', default=False)
    parser.add_argument('--web-models',
                        action='store_true', help='web models download', default=False)

    return parser


# main
if __name__ == '__main__':
    # trajectory link
    # http://iimcb.genesilico.pl/SimRNAweb/media/jobs/
    # rp12aawlpk-8713ed35/processing_results/rp12aawlpk-8713ed35_ALL.trafl

    parser = get_parser()
    args = parser.parse_args()
    job_id = args.job_id
    job_id = job_id.replace('genesilico.pl/SimRNAweb/jobs/', '').replace(
        'http://', '').replace('/', '')  # d86c07d9-9871-4454-bfc6-fb2e6edf13fc/

    if args.web_models or args.top100 or args.top200 or args.download_trajectory:
        download_models(args)  # models are needed if you want to extract anything from the trajectory

    # trajectory
    if args.top100:
        download_trajectory_top100()

    if args.top200:
        download_trajectory_top200()

    if args.download_trajectory:
        download_trajectory()

    if args.cluster:
        copy_trajectory(args)

    if (args.top100 or args.top200):
        extract(args.job_id, args.nstruc, args.remove_trajectory, lowest=False)

    if (download_trajectory or args.cluster) and args.nstruc:
        extract(args.job_id, args.nstruc, args.remove_trajectory, lowest=True)

    if args.extract:
        extract(args.job_id, args.nstruc, args.remove_trajectory)

    if args.prefix:
        add_prefix(args.prefix)

    #if not args.nolog:
    if 1:
        logging.basicConfig(filename='rna_simrnaweb_download_job.log',level=logging.INFO)
        #logging.basicConfig(level=logging.INFO)
        logging.info(time.strftime("%Y-%m-%d %H:%M"))
        logging.info(args)
